/* MODIFIED from example 10-4: code from O'Reilly Vol 6 - Motif manual */

# define XY_SLICE 1
# define XZ_SLICE 2
# define YZ_SLICE 3
# define RZ_SLICE 4
# define AZ_SLICE 5
# define ARB_SLICE 6

#include <Xm/DialogS.h>
#include <Xm/MainW.h>
#include <Xm/TextF.h>
#include <Xm/DrawingA.h>
#include <Xm/PushBG.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleBG.h>
#include <Xm/CascadeB.h>
#include <Xm/ScrolledW.h>
#include <Xm/Form.h>
#include <Xm/ArrowBG.h>
#include <Xm/LabelG.h>
#include <Xm/PanedW.h>
#include <Xm/FileSB.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h> 

# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include <math.h>

# include "xplot_arrows.h"
# include "wind_functions.h"

# define IMAX 400
# define JMAX 400
# define KMAX 200
# define RMAX 400 /* for array to hold rz or arbitrary slices */
# define im_ar_max 500

struct hdr {
	char keywd[4];
	char flight[8];
	char storm[12];
	char radarid[4];
	char trkfil[32];
	char create_time[32];
	char namram[28];
	int idim;
	int jdim;
	int kdim; 
	int nsweeps;
	int nmosm;
	int unfold;
	int interven_attn;
	int flag_dop;
	int iextra2;
	float extra3; 
	float stime1; 
	float etime2;
	float olat; 
	float olon;
	float sxt;
	float syt;
	float szt;
	float xz;
	float yz;
	float zz;
	float rot;
	float ra;
	float co1;
	float co2;
	float azmcor;
	float elcor;
	float thresh;
	float powerT;
	float pcor;
	float dcor;
	float rcor;
	float extra5;
	float extra6;
	float extra7;
};

# define REFLECTIVITY 1
# define WINDSPEED 2
# define VERTWINDSPEED 3
# define TANWIND 4
# define RADWIND 5
# define U_COMPONENT 6
# define V_COMPONENT 7
struct chitra {
	int idim, jdim, kdim; /* actual dimensions of data array */
	int var_type;
	int level;
	int orientation; /* 1-xy, 2-xz, 3-yz */
	int short data[KMAX][JMAX][IMAX][5];
	struct hdr *head;	
	int shift_center; /* flag to shift center of circulation */
	float cen_shift[2]; /* (x,y) for shifting circulation center */
	int remove_motion; /* flag to remove storm motion */
	int xy_arrows; /* plot arrows (1) or wind barbs (0) in xy slices */
	float storm_uv[2]; /* for subtracting constant velocity */
	float w_exag; /* for exagerating w for wind barb overlays */
	float var_scale[8][2]; /* Variable min and max for color scaling */
	float rad_min, rad_max; /* distance from storm, for RZ means */
	XImage *xi;
	XFontStruct *font_str; /* hold info about current font, for labels*/
	int width, height; /* width and height of total drawing area */
	int pic_wid, pic_ht, pic_left, pic_bottom; /* sub window */
	unsigned char image_array[im_ar_max][im_ar_max]; /*holds pixel image */
	Widget widget; /* where I put drawing_a temporarily */
	float arc_min_frac; /* percent of arc that must be filled in RZ mean*/
	int iw[2], jw[2], kw[2]; /* array indexes for sub_window */
	int nzx, nzy; /* for mapping from screen back to array */
	int arrow_shaft_width; /* for calls to XSetLineWidth */
	int arrow_shaft_length; /* to set length of arrows */
	int arrow_interval; /* how often to plot, 1 = every pt, etc. */
	float az_min, az_max; /* for az - height displays */
	float x_beg, y_beg, x_end, y_end; /* for arbitrary slices */
};
struct chitra flic;

GC gc;	/* Graphics context..needed for Xlib calls */

Pixmap pixmap;
/* dimensions of drawing area (pixmap) */
Dimension width, height;

String colors[] = {
/* My attempt to reproduce m2.col from Sigmet, using colors in 
 * /usr/lib/X11/rgb.txt. However I used Purple instead of Brown
 * to get better contrast. This could be modified to read in color
 * table in RGB format, so I am not stuck with X11 defaults.
 */

/*
 */
   	"Black", "Gray38", "Navy", "DodgerBlue4", "DodgerBlue2",
	"DarkGreen", "LimeGreen", "MediumSpringGreen", "Yellow",
	"DarkOrange2", "SaddleBrown", "Purple", "OrangeRed3",
	"Red", "HotPink1", "White"


/* For making conference figures, a gray scale.. 
 
 	"Black", "Gray91", "Gray84", "Gray77", "Gray70",
 	"Gray63","Gray56","Gray49","Gray42",
 	"Gray35","Gray28","Gray21","Gray14",
 	"Gray7","Gray1","White"
 */
};

unsigned long col_pix[16];


Widget xy, xz, yz, rz, az_z, arb_sl; /* GLOBAL so radio box stuff can query */
Widget ws_wdgt, dbz_wdgt, tans_wdgt, rads_wdgt, vertv_wdgt, u_wdgt, v_wdgt;
Widget lev_label; /* GLOBAL so set_level can change label in arrow box */
Widget shift_label, motion_label; /* GLOBAL so can set change labels */

main( argc, argv)
int argc;
char *argv[];
{
	Widget toplevel, main_w, sw, menu, bar, form, drawing_a, pb;
	Widget sl_menu, da_menu; /* RowColumn widgets to hold radio boxes */
	Widget orien_rb; /* RadioBoxes for orientation and data type */
	Widget datype_rb;
	Widget ar_box, pb_up, pb_down; /* RC box and Arrows */
	Widget correct_menu, scale_menu;
	XtAppContext app;
	XGCValues gcv;
	Colormap cmap;
/*        Colormap cmap2; */
	XColor col, unused;
	Visual *visual;	
	XFontStruct *font_info;
	
	void read_file(), file_cb();
	void draw(), redraw(), set_color(), exit(), clear_it();
	void set_orien(), set_datype(), set_level(),PaintLevel();
	void incr_level(), decr_level();
	void shift_center(), rem_motion();
	void value_scaling(), geometry_scaling(), arrow_scaling();
	void set_ring_frac();
	void WindOverlay(), Caption();
	int load_font();
	void get_rmin();
	void mouser();
					
	XtActionsRec actions;
	String translations = /* for the DrawingArea widget */
		" <Btn1Down>: draw(down)\n\
		  <Btn1Up>:   draw(up)	\n\
		  <Btn1Motion>: draw(motion)";
		  
	struct hdr radar_hdr;
	extern struct chitra flic;
/* Stuff for my attempt to homebrew an image: */
	unsigned int im_depth, im_wid, im_ht;
	int im_format, im_offset, im_bmap_pad, im_bytes_per_line;
	
 	FILE *infile;
	char string [32], filename[81];

	int i, j, k, level = 0,  max_x, max_y;

	if( argc < 2 ) {
/*		printf(" Enter 3d file name:\n");
 *		gets(filename);	
 */
		printf(" use file requester in Stuff.\n");
	}
	else {
		strcpy(filename, argv[1]);
		printf(" Will try to open %s.\n",filename);
		infile = fopen( filename, "r");
		if ( infile == NULL ) {
			printf("Can't open file %s\n", filename);
			exit(0);
		}
/* following func, also called by file selection stuff, reads in hdr and
 * data and loads pointers to them in structure flic */

                printf("calling read_file \n");
		read_file( infile, &flic );
                printf("returned from read_file \n");
		/*		fclose( infile); */
                printf("finished fclose \n");
	}
	
/* Default parts of structure flic that holds info for drawing
 * set so program won't blow up if we start drawing before picking anything!
 */
	flic.var_type = REFLECTIVITY;
	flic.var_scale[REFLECTIVITY][0] = 0.0;
	flic.var_scale[REFLECTIVITY][1] = 64.0;
	flic.var_scale[WINDSPEED][0] = 0.0;
	flic.var_scale[WINDSPEED][1] = 75.0;
	flic.var_scale[TANWIND][0] = 0.0;
	flic.var_scale[TANWIND][1] = 75.0;
	flic.var_scale[RADWIND][0] = -20.0;
	flic.var_scale[RADWIND][1] = 20.0;
	flic.var_scale[VERTWINDSPEED][0] = -15.0;
	flic.var_scale[VERTWINDSPEED][1] = 15.0;
	flic.var_scale[U_COMPONENT][0] = -30.0;
	flic.var_scale[U_COMPONENT][1] = 30.0;
	flic.var_scale[V_COMPONENT][0] = -30.0;
	flic.var_scale[V_COMPONENT][1] = 30.0;
	flic.level = 0;
	flic.orientation = 1;
	flic.cen_shift[0] = 0.0;
	flic.cen_shift[1] = 0.0;
	flic.shift_center = 0;
	flic.storm_uv[0] = 0.0;
	flic.storm_uv[1] = 0.0;
	flic.remove_motion = 0;
	flic.arc_min_frac = 0.33;
	flic.w_exag = 1.0; /* This will be reset in PaintLevel */
	flic.xy_arrows = 1; /* xy =1 will plot arrows, else barbs */
	flic.arrow_shaft_width = 0; /* fastest arrow drawing */
	flic.arrow_shaft_length = 50; /* arrow length for 50 m/s */
	flic.arrow_interval = 2; /* plot every other arrow */
	toplevel = XtVaAppInitialize(&app, "RadarSlicer", NULL, 0,
		&argc, argv, NULL, NULL);
		
	/* Create MainWindow to contain drawing area */
	main_w = XtVaCreateManagedWidget("main_w",
		xmFormWidgetClass, toplevel, NULL);
		
	/* Create a GC for drawinf (callback). used alot so make global */
	gcv.foreground = WhitePixelOfScreen( XtScreen(main_w) ) ;
	gc = XCreateGC(XtDisplay(main_w),
		RootWindowOfScreen(XtScreen(main_w)), GCForeground, &gcv);

	/* Create menu bar */
	bar = XmCreateMenuBar(main_w, "menubar", NULL, 0 );
	XtVaSetValues( bar,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);
	XtManageChild(bar);
	
/* Create file selction and quit menu */
	menu = XmCreatePulldownMenu( bar,"stuff", NULL, 0);
	XtVaCreateManagedWidget("Stuff", xmCascadeButtonWidgetClass,
		bar, XmNsubMenuId, menu, NULL);
	pb = XtVaCreateManagedWidget("file", xmPushButtonWidgetClass, 
		menu, NULL);
	XtAddCallback(pb, XmNactivateCallback, file_cb, &flic);

	pb = XtVaCreateManagedWidget("quit", xmPushButtonWidgetClass, 
		menu, NULL);
	XtAddCallback(pb, XmNactivateCallback, (XtCallbackProc) exit, NULL);

/* Create corrections menu */
	correct_menu = XmCreatePulldownMenu( bar,"correct", NULL, 0);
	XtVaCreateManagedWidget("Correct", xmCascadeButtonWidgetClass,
		bar, XmNsubMenuId, correct_menu, NULL);
	pb = XtVaCreateManagedWidget("Ring Frac", xmPushButtonWidgetClass, 
		correct_menu, NULL);
	XtAddCallback(pb, XmNactivateCallback, set_ring_frac, &flic);
	pb = XtVaCreateManagedWidget("Center", xmPushButtonWidgetClass, 
		correct_menu, NULL);
	XtAddCallback(pb, XmNactivateCallback, shift_center, &flic);
	pb = XtVaCreateManagedWidget("Motion", xmPushButtonWidgetClass, 
		correct_menu, NULL);
	XtAddCallback(pb, XmNactivateCallback, rem_motion, &flic);

/* Create scaling menu */
	scale_menu = XmCreatePulldownMenu( bar,"scale", NULL, 0);
	XtVaCreateManagedWidget("Scale", xmCascadeButtonWidgetClass,
		bar, XmNsubMenuId, scale_menu, NULL);
	pb = XtVaCreateManagedWidget("Values", xmPushButtonWidgetClass, 
		scale_menu, NULL);
	XtAddCallback(pb, XmNactivateCallback, value_scaling, &flic);
	pb = XtVaCreateManagedWidget("Geometry", xmPushButtonWidgetClass, 
		scale_menu, NULL);
	XtAddCallback(pb, XmNactivateCallback, geometry_scaling, &flic);
	pb = XtVaCreateManagedWidget("Arrows", xmPushButtonWidgetClass, 
		scale_menu, NULL);
	XtAddCallback(pb, XmNactivateCallback, arrow_scaling, &flic);

/* Create a widget to hold two arrows and label, for level selections 
 * Using code from O'Reilly vol 6, p 380 */
	ar_box = XtVaCreateWidget("ar_box", xmRowColumnWidgetClass,
		main_w, XmNorientation, XmHORIZONTAL, 
		XmNtopAttachment,	XmATTACH_WIDGET,
		XmNtopWidget,		bar,
		XmNleftAttachment,	XmATTACH_FORM,
		NULL);
	pb_up = XtVaCreateManagedWidget("pb_up",xmArrowButtonGadgetClass,
		ar_box, XmNarrowDirection, XmARROW_UP, NULL);
	XtAddCallback(pb_up, XmNactivateCallback, incr_level, &flic);
	lev_label = XtVaCreateManagedWidget("lev_label",
		xmLabelGadgetClass, ar_box, 
		XtVaTypedArg, XmNlabelString, XmRString,"nn.n km ",8,
		NULL);
	pb_down = XtVaCreateManagedWidget("pb_down",xmArrowButtonGadgetClass,
		ar_box, XmNarrowDirection, XmARROW_DOWN, NULL);
	XtAddCallback(pb_down, XmNactivateCallback, decr_level, &flic);
	pb = XtVaCreateManagedWidget("show", xmPushButtonWidgetClass, 
		ar_box, NULL);
	XtAddCallback(pb, XmNactivateCallback, PaintLevel, &flic);

	pb = XtVaCreateManagedWidget("Overlay", xmPushButtonWidgetClass, 
		ar_box, NULL);
/*        printf("Testing Printf"); */
	XtAddCallback(pb, XmNactivateCallback, WindOverlay, &flic);
	XtManageChild(ar_box);
		

/* Next create another widget to hold  radio boxes for orientation,
 * using example code from O'Reilly vol 6, p 406. */
	sl_menu = XtVaCreateWidget("slices", xmRowColumnWidgetClass, main_w,
		XmNnumColumns,		1,
		XmNorientation,		XmHORIZONTAL,
		XmNpacking,		XmPACK_COLUMN,
		XmNtopAttachment,	XmATTACH_WIDGET,
		XmNtopWidget,		bar,
		XmNleftAttachment,	XmATTACH_WIDGET,
		XmNleftWidget,		ar_box,
		NULL);
 	orien_rb = XmCreateRadioBox(sl_menu, "Orientation",
		NULL, 0);
	XtVaSetValues(orien_rb, XmNorientation, XmHORIZONTAL, NULL);
		
	xy = XtVaCreateManagedWidget("XY",
		xmToggleButtonGadgetClass, orien_rb, NULL);	
	XtAddCallback(xy, XmNvalueChangedCallback, set_orien, &flic);
	xz = XtVaCreateManagedWidget("XZ",
		xmToggleButtonGadgetClass, orien_rb, NULL);	
	XtAddCallback(xz, XmNvalueChangedCallback, set_orien, &flic);
	yz = XtVaCreateManagedWidget("YZ",
		xmToggleButtonGadgetClass, orien_rb, NULL);	
	XtAddCallback(yz, XmNvalueChangedCallback, set_orien, &flic);
	rz = XtVaCreateManagedWidget("RZ",
		xmToggleButtonGadgetClass, orien_rb, NULL);	
	XtAddCallback(rz, XmNvalueChangedCallback, set_orien, &flic);
	az_z = XtVaCreateManagedWidget("AZ-Z",
		xmToggleButtonGadgetClass, orien_rb, NULL);	
	XtAddCallback(az_z, XmNvalueChangedCallback, set_orien, &flic);
	arb_sl = XtVaCreateManagedWidget("ARB",
		xmToggleButtonGadgetClass, orien_rb, NULL);	
	XtAddCallback(arb_sl, XmNvalueChangedCallback, set_orien, &flic);
	XtManageChild(orien_rb);

	XtManageChild(sl_menu);
/* Next create radio box for data types
 */
	da_menu = XtVaCreateWidget("data type", xmRowColumnWidgetClass, 
		main_w,
		XmNnumColumns,		2,
		XmNorientation,		XmHORIZONTAL,
		XmNpacking,		XmPACK_COLUMN,
		XmNtopAttachment,	XmATTACH_WIDGET,
		XmNtopWidget,		sl_menu,
		XmNleftAttachment,	XmATTACH_FORM,
		NULL);

	datype_rb = XmCreateRadioBox(da_menu, "Data",
		NULL, 0);
	XtVaSetValues(datype_rb, XmNorientation, XmHORIZONTAL, NULL);
	
	dbz_wdgt = XtVaCreateManagedWidget(" Reflectivity ",
		xmToggleButtonGadgetClass, datype_rb, NULL);	
	XtAddCallback(dbz_wdgt, XmNvalueChangedCallback, set_datype, &flic);
	ws_wdgt = XtVaCreateManagedWidget(" Wind speed ",
		xmToggleButtonGadgetClass, datype_rb, NULL);	
	XtAddCallback(ws_wdgt, XmNvalueChangedCallback, set_datype, &flic);
	tans_wdgt = XtVaCreateManagedWidget(" Tan. Wind ",
		xmToggleButtonGadgetClass, datype_rb, NULL);	
	XtAddCallback(tans_wdgt, XmNvalueChangedCallback, set_datype, &flic);
	rads_wdgt = XtVaCreateManagedWidget(" Radial Wind",
		xmToggleButtonGadgetClass, datype_rb, NULL);	
	XtAddCallback(rads_wdgt, XmNvalueChangedCallback, set_datype, &flic);
	vertv_wdgt = XtVaCreateManagedWidget(" Vert Wind",
		xmToggleButtonGadgetClass, datype_rb, NULL);	
	XtAddCallback(vertv_wdgt, XmNvalueChangedCallback, set_datype, &flic);
	u_wdgt = XtVaCreateManagedWidget("U",
		xmToggleButtonGadgetClass, datype_rb, NULL);	
	XtAddCallback(u_wdgt, XmNvalueChangedCallback, set_datype, &flic);
	v_wdgt = XtVaCreateManagedWidget("V",
		xmToggleButtonGadgetClass, datype_rb, NULL);	
	XtAddCallback(v_wdgt, XmNvalueChangedCallback, set_datype, &flic);
	XtManageChild(datype_rb);
	XtManageChild(da_menu);	

/* Now display area stuff:
 */

	sw = XtVaCreateManagedWidget(" scrolled_win ",
		xmScrolledWindowWidgetClass, 	main_w,
		XmNwidth,			300,
		XmNscrollingPolicy,		XmAUTOMATIC,
		XmNscrollBarDisplayPolicy,	XmAS_NEEDED,
		XmNtopAttachment,		XmATTACH_WIDGET,
		XmNtopWidget,			da_menu,
		XmNbottomAttachment,		XmATTACH_FORM, 
		XmNleftAttachment,		XmATTACH_FORM,
		XmNrightAttachment,		XmATTACH_FORM,
		NULL);
		
	/* Add the draw action/function used by the translation table
	 * parsed by the translations resource below */
	actions.string = "draw";
	actions.proc = draw;
	XtAppAddActions(app, &actions, 1);
	
/* Create a DrawingArea widget, 9" wide by 7" tall.
 * Do not let it resize, so clear button does not force resize */
 
	drawing_a = XtVaCreateManagedWidget("drawing_a",
		xmDrawingAreaWidgetClass, sw,
/* commented out 18 aug 97, so mouser will deal with button clicks in window
		XmNtranslations, XtParseTranslationTable(translations),
 */
		XmNunitType,	Xm1000TH_INCHES,
		XmNwidth,	9000,
		XmNheight,	7000,
		XmNresizePolicy,  XmNONE, 
		NULL);
		
	/* When scrolled, the drawing area will get expose events */
	XtAddCallback(drawing_a, XmNexposeCallback, redraw, NULL);
/* note from text..." The Clear button's callback function was passed
 * the DrawingArea widget as the client data when the callback was installed.
 * This saved us from having to declare a global variable, yet still makes
 * the handle to the DrawingArea available to the button's callback routine"
 * page 353, July 1992 edition */
 
	/* pushing clear button clears drawing area widget */
	pb = XtVaCreateManagedWidget("clear", xmPushButtonWidgetClass, 
		ar_box, NULL);
	XtAddCallback(pb, XmNactivateCallback, clear_it, drawing_a);

/* Add event handler to handle mouse button stuff in drawing_a.
 * Follow example Code from Vol IV, Xt Intrinsics Programming Manual
 * Example 9-1, p. 268
 */
/* Note: I added PointerMotionMask so that Window Edit mode will get
 * pointer location even if mouse button not pressed... may need tweaking
 * if it uses too much resources that way 11 sept 1995 */
 	XtAddEventHandler(drawing_a,
	 PointerMotionMask|ButtonPressMask|ButtonReleaseMask|ButtonMotionMask, /* Events */
 		False,		/* non-maskable events */
 		mouser,		/* event handler function */
 		&flic);		/* my data structure */

/* convert drawing area back to pixels to get width and height */
	XtVaSetValues(drawing_a, XmNunitType, XmPIXELS, NULL);
	XtVaGetValues(drawing_a, XmNwidth, &width, XmNheight, &height, NULL);

/*create a pixmap the same size as drawing area */
	pixmap = XCreatePixmap(XtDisplay(drawing_a), 
	  RootWindowOfScreen(XtScreen(drawing_a)), width, height,
	  DefaultDepthOfScreen(XtScreen(drawing_a)) );
	/* clear pixmap with white */
	set_color(drawing_a, "White");
	XFillRectangle(XtDisplay(drawing_a), pixmap, gc, 0, 0, width, height);  
	set_color(drawing_a, "Black"); /* so draw will do something! */

	if( load_font(XtDisplay(main_w), &font_info) != -1) {
		XSetFont(XtDisplay(main_w), gc, font_info->fid);
		flic.font_str = font_info;
	}
	else {
		printf(" Trouble loading font. Will use default.\n");
		font_info = XQueryFont( XtDisplay(main_w), 
			XGContextFromGC( gc ) );
/*		XSetFont(XtDisplay(main_w), gc, font_info->fid); */
		flic.font_str = font_info;
	}
	flic.width = width;
	flic.height = height;
/* Leave room for 7 characters on the left ...*/
	sprintf(string, "1234567\n");
	flic.pic_left = XTextWidth(font_info, string, strlen(string));
/* and 15 on the right */
	sprintf(string, "123456789012345\n");
 	flic.pic_wid = width - flic.pic_left -
		XTextWidth(font_info, string, strlen(string));
/* Leave room for 6 strings (3 on top 3 on bottom) */
 	flic.pic_ht = height - 6.0 * 
 		(font_info->ascent + font_info->descent);
 	flic.pic_bottom = height - flic.pic_ht - 3.0 * 
 		(font_info->ascent + font_info->descent);;	
/* Maintain square window... */
 	if( flic.pic_wid > flic.pic_ht )
 		flic.pic_wid = flic.pic_ht;

	flic.widget = drawing_a;

/* Now get image data structure
 * For first try, use getimage and xputpixel; maybe later optimize by setting
 * own image data array and writing directly to it.
 */

	printf(" Attempting to create image \n");
/* Try using Image stuff */
	im_wid = im_ar_max;
	im_ht = im_ar_max;
	im_depth = DefaultDepthOfScreen(XtScreen(drawing_a));
	visual = DefaultVisual(XtDisplay(drawing_a),
		RootWindowOfScreen(XtScreen(drawing_a)) );
	printf(" Screen depth: %d \n",im_depth);
	im_format = ZPixmap; 
	im_offset = 0;
	im_bmap_pad = XBitmapPad(XtDisplay(drawing_a) ); 
	im_bytes_per_line = 0; /* let Xlib figure it out */
/* This code does not work at present
	printf(" Calling XCreateImage with these arguments: \n");
	printf(" im_depth %d, im_format %d, im_offset %d \n",
		im_depth, im_format, im_offset );
	printf(" im_wid %d, im_ht %d, im_bmap_pad %d, im_bytes_per_line %d \n",
		im_wid, im_ht, im_bmap_pad, im_bytes_per_line );
	flic.xi = XCreateImage(XtDisplay(drawing_a), visual, im_depth,
		im_format, im_offset, (char *) NULL, im_wid, im_ht,
		im_bmap_pad, im_bytes_per_line);
 * so instead we use XGetImage */
 	printf(" Calling XGetImage with these arguments: \n");
	printf(" im_format %d \n", im_format);
	flic.xi = XGetImage(XtDisplay(drawing_a), pixmap, 0,0,
		width, height,
		AllPlanes,im_format);

	if( flic.xi == NULL )
		printf(" XGetImage failed...\n");
	else
		printf(" XGetImage ok. bits_per_pixel = %d \n",
			flic.xi->bits_per_pixel);
/* load color lut */
	cmap = DefaultColormapOfScreen(XtScreen(toplevel));
/*      cmap2 = XCopyColormapAndFree(XtDisplay(drawing_a),cmap);
	flic.xi = XGetImage(XtDisplay(drawing_a), pixmap, 0,0,
		width, height,
		AllPlanes,im_format);

	if( flic.xi == NULL )
		printf(" XGetImage failed...\n");
	else
		printf(" XGetImage ok. bits_per_pixel = %d \n",
			flic.xi->bits_per_pixel);*/
        printf("XtNumber(colors) is %d \n",XtNumber(colors));
	for (i = 0; i < XtNumber(colors) ; i++) {
          printf("i = %d \n",i);
	    if( !XAllocNamedColor(XtDisplay(drawing_a), cmap, colors[i], 
	    	&col, &unused) ) {
		    printf("Can't alloc %s \n",colors[i]);
		    col_pix[i] = 0;
	    }
	    else
	    	{col_pix[i] =  col.pixel; 
                printf("col.pixel = %d \n",col.pixel);}
	}

	XtRealizeWidget( toplevel);
        printf("I got here \n");
	XtAppMainLoop(app);
}

/* Action procedure to respond to any of the events from the
 * translation table declared in main(). This function is called 
 * in response to Button1 Down, Up and Motion events. Basically,
 * we're just doing a freehand draw -- not lines or anything. */
 
void  draw(widget, event, args, num_args)
Widget widget;
XButtonEvent *event;
String *args;
int *num_args;
{
	static Position x, y;
	if (*num_args != 1)
		XtError("Wrong number of args!");
		
	if( strcmp(args[0], "down") ) {
		/* if it is not down, it must be either "up" or
		 * "motion". Draw a full line from anchor point to new point.
		 */
		XDrawLine(event->display, event->window, gc, x, y, 
			event->x, event->y);
		XDrawLine(event->display, pixmap, gc, x, y, 
			event->x, event->y);
	}
	/* freehand is really a bunch of line segments */		
	x = event->x;
	y = event->y;
}

/* Clear the window by clearing the pixmap and calling XCopyArea() */
void clear_it(pb, drawing_a, cbs)
Widget pb, drawing_a;
XmPushButtonCallbackStruct *cbs;
{
	/* clear pixmap with white */
	XSetForeground(XtDisplay(drawing_a), gc, 
		WhitePixelOfScreen(XtScreen(drawing_a)) );
	/* this clears the pixmap */
	XFillRectangle(XtDisplay(drawing_a), pixmap, gc, 0, 0, width, height);
	/* drawing is now done using black, change the gc */
	XSetForeground(XtDisplay(drawing_a), gc, 
		BlackPixelOfScreen(XtScreen(drawing_a)) );
	/* render the newly cleared pixmap into the window */
	XCopyArea(cbs->event->xbutton.display, pixmap, XtWindow(drawing_a), 
		gc, 0, 0, width, height, 0, 0);
}

/* redraw is called whenever all or portions of the drawing area are exposed.
 * This includes newly exposed portions of the widget resulting from the 
 * users interaction with the scroll bars.
 */
void redraw(drawing_a, client_data, cbs)
Widget	drawing_a;
XtPointer client_data;
XmDrawingAreaCallbackStruct *cbs;
{
	XCopyArea(cbs->event->xexpose.display, pixmap, cbs->window,
		gc, 0, 0, width, height, 0, 0);
}

/* callback routine for when any of the color tiles are pressed.
 * This general function may also be used to set the global gc's
 * color directly. Just provide a widget and a color name.
 */
void set_color(widget, color)
Widget widget;
String color;
{
	Display *dpy = XtDisplay(widget);
	Colormap cmap = DefaultColormapOfScreen(XtScreen(widget));
	XColor col, unused;
	if( !XAllocNamedColor(dpy, cmap, color, &col, &unused) ) {
		char buf[32];
		sprintf(buf, "Can't alloc %s",color);
		XtWarning(buf);
		return;
	}
	XSetForeground(dpy, gc, col.pixel);
}

/* ARGSUSED */	
void set_level(new_level)
float new_level;
/* Change label with latest slice, in km */
{
	char buf[8];
	sprintf(buf, "%.1f km ", new_level);
	XtVaSetValues( lev_label, XtVaTypedArg, XmNlabelString, 
		XmRString, buf, strlen(buf), NULL);
	return;
}

/* ARGSUSED */	
void incr_level(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;
{
	void set_level();
	/* cast required in ANSI C , but I could not get to work if
	 * cast the way they did in O'Reilly....*/
	struct chitra *flic = client_data;
	float lev_km;
/* Increment level so next button push loads and displays next level up*/
	if( flic->orientation == XY_SLICE ) {	/* xy slice */
		flic->level = flic->level + 1;
		if(flic->level >= flic->kdim)
			flic->level = 0;
		lev_km = flic->head->szt * (flic->level +1);
	}
	else if( flic->orientation == XZ_SLICE ) { /* xz slice */
		flic->level = flic->level + 1;
		if(flic->level >= flic->jdim)
			flic->level = 0;
		lev_km = flic->head->syt * (flic->level +1);
	}
	else 	if( flic->orientation == YZ_SLICE ) { /*yz slice */
		flic->level = flic->level + 1;
		if(flic->level >= flic->idim)
			flic->level = 0;
		lev_km = flic->head->sxt * (flic->level +1);
	}
	else 	if( flic->orientation == AZ_SLICE ) { /*azimuth - height slice */
		flic->level = flic->level + 1;
		if( flic->level > flic->rad_max )
			flic->level = flic->rad_min;
		else if( flic->level < flic->rad_min )
                        flic->level = flic->rad_max;
		lev_km = flic->level;
	}
	set_level(lev_km); /* put new level on lev_label widget */
/*	printf(" Will slice level %d \n", flic->level); */
	return;
}

void decr_level(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;
{
	void set_level();
	/* cast required in ANSI C , but I could not get to work if
	 * cast the way they did in O'Reilly....*/
	struct chitra *flic = client_data;
	float lev_km;
/* Decrement level so next button push loads and displays next level down*/
	if( flic->orientation == XY_SLICE ) {	/* xy slice */
		flic->level = flic->level - 1;
		if(flic->level < 0)
			flic->level = flic->kdim-1;
		else if(flic->level >= flic ->kdim)
			flic->level = 0;
		lev_km = flic->head->szt * (flic->level +1);
	}
	else if( flic->orientation == XZ_SLICE ) { /* xz slice */
		flic->level = flic->level - 1;
		if(flic->level < 0)
			flic->level = flic->jdim -1;
		else if(flic->level >= flic->jdim)
			flic->level = 0;
		lev_km = flic->head->syt * (flic->level +1);
	}
	else 	if( flic->orientation == YZ_SLICE ) { /*yz slice */
		flic->level = flic->level - 1;
		if(flic->level < 0 )
			flic->level = flic->idim - 1;
		else if(flic->level >= flic->idim)
			flic->level = 0;
		lev_km = flic->head->sxt * (flic->level +1);
	}
	else 	if( flic->orientation == AZ_SLICE ) { /*azimuth - height slice */
		flic->level = flic->level - 1;
		if( flic->level < flic->rad_min )
			flic->level = flic->rad_max;
		else if (flic->level > flic->rad_max)
			flic->level = flic->rad_min;
		lev_km = flic->level;
	}
/*	printf(" Will slice level %d \n", flic->level);*/
	set_level(lev_km); /* put new level on lev_label widget */
	return;
}

/* ARGSUSED */	
void set_orien(w, client_data, state)
Widget w;
XtPointer client_data;
XmToggleButtonCallbackStruct *state;

{ 
	/* cast required in ANSI C , but I could not get to work if
	 * cast the way they did in O'Reilly....*/
	struct chitra *flic = client_data;
	int i, j;
	unsigned long pixie;
	
	if( state->set) {
/* This is crude... test each toggle to see which was set, and set 
 * flic->orientation accordingly.. to do this xy, xz, yz are GLOBAL widgets,
 * which i dont like but ke garne
 */
		if( XmToggleButtonGetState(xy) )
			flic->orientation = XY_SLICE;
		else if( XmToggleButtonGetState(xz) )
			flic->orientation = XZ_SLICE;
		else if( XmToggleButtonGetState(yz) )
			flic->orientation = YZ_SLICE;
		else if( XmToggleButtonGetState(rz) )
			flic->orientation = RZ_SLICE;
		else if( XmToggleButtonGetState(az_z) )
			flic->orientation = AZ_SLICE;
		else if( XmToggleButtonGetState(arb_sl) )
			flic->orientation = ARB_SLICE;
/*		printf("selected %d orientation \n",flic->orientation); */
	}
	else	flic->orientation = XY_SLICE;  /* default */
/* Zero out image, because vertical and horizontal slices do not 
 * take up quite the same space. 
 */
	pixie = col_pix[15]; /* white out... */
	for( j = flic->pic_bottom; j < (flic->pic_bottom + flic->pic_ht); j++)
	   for( i = flic->pic_left; i < (flic->pic_left+flic->pic_wid); i++)
		XPutPixel(flic->xi, i, flic->height - j, pixie);

	return;
}

/* ARGSUSED */	
void set_datype(w, client_data, state)
Widget w;
XtPointer client_data;
XmToggleButtonCallbackStruct *state;

{ 
	struct chitra *flic = client_data;
	if( state->set) {
/* This is crude... test each toggle to see which was set, and set 
 * flic->var_type.
 */
		if( XmToggleButtonGetState(ws_wdgt) )
			flic->var_type = WINDSPEED;
		else if( XmToggleButtonGetState(dbz_wdgt) )
			flic->var_type = REFLECTIVITY;
		else if( XmToggleButtonGetState(tans_wdgt) )
			flic->var_type = TANWIND; 
		else if( XmToggleButtonGetState(rads_wdgt) )
			flic->var_type = RADWIND;
		else if( XmToggleButtonGetState(vertv_wdgt) )
			flic->var_type = VERTWINDSPEED;
		else if( XmToggleButtonGetState(u_wdgt) )
			flic->var_type = U_COMPONENT;
		else if( XmToggleButtonGetState(v_wdgt) )
			flic->var_type = V_COMPONENT;

/*		printf(" Toggled data type %d\n",flic->var_type);*/
	}
	else
	{	
			flic->var_type = REFLECTIVITY;  /* default is dBZ */
/*			printf(" Default data type %d\n",flic->var_type); */
	}
	return;
}

/* ARGSUSED */	
void PaintLevel(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;

{
	/* cast required in ANSI C , but I could not get to work if
	 * cast the way they did in O'Reilly....*/
	struct chitra *flic = client_data;
	void zoom_and_paint(), Caption(), Grid(), ColorScale();
	int i, j, k, kcorn, iv, ir, ia, iwd, iht;
	float zscale, zmin, zmax;
	int zero_contour, zero_color;
	float a11, a12, a21, a22, bad_val, val_scaling;
	float wdir, wspd, u, v, vr, vt;
	float x, y, ysq, xc, yc, zc, radius, az, d_rav, d_az, d_s;
	float rav_array[KMAX][RMAX][2];
	
/*        printf("PaintLevel called"); */
	xc = flic->head->xz; /* position of storm center rel to lower left */
	yc = flic->head->yz;
	zc = flic->head->zz; /* height of first layer in array */
	zmin = flic->var_scale[flic->var_type][0];
	zmax = flic->var_scale[flic->var_type][1];
 	
	if( flic->var_type == REFLECTIVITY)
	{
		iv = 3; /* 4th variable in WIND files */
		val_scaling = 0.1;
		bad_val = -900.0;
	}
	else if( flic->var_type == WINDSPEED)
	{
		iv = 1;
		val_scaling = 0.1;
		bad_val = -900.0;
	}
	else if( flic->var_type == VERTWINDSPEED)
	{
		iv = 2;
		val_scaling = 0.01;
		bad_val = -90.0;
	}
	else if( flic->var_type == RADWIND) {
		iv = -1; /* which means convert wd, ws to u,v to vtan, vrad */
		bad_val = -900.0;
	}
	else if( flic->var_type == TANWIND) {
		iv = -2; /* convert wd, ws to u,v to vtan, vrad */
		bad_val = -900.0;
	}
	else if( flic->var_type == U_COMPONENT) {
		iv = -3; /* convert wd, ws to u,v */
		bad_val = -900.0;
	}
	else if( flic->var_type == V_COMPONENT) {
		iv = -4; /* convert wd, ws to u,v */
		bad_val = -900.0;
	}
	else if (flic->var_type < 1 )
	{
		printf(" Data type %d not implemented.\n",flic->var_type);
		return;
	}
	zscale = (XtNumber(colors)-1) / (zmax - zmin); 

/* 	zscale = 255 / (zmax-zmin);
 *
 *	zscale = 63 / (zmax-zmin);  
 */
 
/* Clear the pixmap before drawing or redrawing */
/* Note that we clear XImage when orientation is reset */
	set_color(flic->widget, "White");
	XFillRectangle(XtDisplay(flic->widget), pixmap, gc, 0, 0, 
		flic->width, flic->height);  
	
	zero_contour = 0; /* =1 means set zero contour...does not work yet */
	zero_color = 15; /* should be white! */
/* set up window, leaving space for grid, labels, etc... 
 * right now its primitive, but better would be to inquire about
 * size of font and use that to set. 
 */
	if ( flic->orientation == XY_SLICE) { /* XY slices */
	  flic->nzx = flic->pic_wid / flic->idim ;
	  flic->nzy = flic->pic_ht / flic->jdim ;
	  if( flic->nzx < flic->nzy) flic->nzy = flic->nzx; /* square up */
	  flic->nzx = flic->nzy;
	  k = flic->level;
/* Note that because C array sized idim are indexed from 0 to idim-1, and 
 * inside loop i address i+1, j+1, for loop has to halt at idim - 2, etc.
 */
	  for(j = 0; j < flic->jdim-1 ; j++ ) {
	    for (i = 0; i < flic->idim-1; i++) {
	    	if( (flic->var_type == REFLECTIVITY) || 
	    	    (flic->var_type == VERTWINDSPEED) ) {
			a12 = flic->data[k][j+1][i][iv] * val_scaling;
			a11 = flic->data[k][j][i][iv] * val_scaling;
			a22 = flic->data[k][j+1][i+1][iv] * val_scaling;
			a21 = flic->data[k][j][i+1][iv] * val_scaling;
		}
		else {	/* convert wd, ws to u, v and then to Vtan, Vrad */
			/* i know following repetition is ugly, but will
			 * make a function after i get working */
			wdir = flic->data[k][j][i][0] * 0.1;
			wspd = flic->data[k][j][i][1] * 0.1;
			x = i * flic->head->sxt;
			y = j * flic->head->syt;
			if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1)
			{
				if( flic->var_type == RADWIND )a11 = vr;
				else if( flic->var_type == TANWIND )a11 = vt;
				else if( flic->var_type == U_COMPONENT )
					a11 = u;
				else if( flic->var_type == V_COMPONENT )
					a11 = v;
				else if( flic->var_type == WINDSPEED )
					a11 = sqrt(u*u + v*v);
			}
			else {
				a11 = bad_val;
			}
			wdir = flic->data[k][j+1][i][0] * 0.1;
			wspd = flic->data[k][j+1][i][1] * 0.1;
			x = i * flic->head->sxt;
			y = (j+1) * flic->head->syt;
			if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1)
			{
				if( flic->var_type == RADWIND )a12 = vr;
				else if( flic->var_type == TANWIND )a12 = vt;
				else if( flic->var_type == U_COMPONENT )
					a12 = u;
				else if( flic->var_type == V_COMPONENT )
					a12 = v;
				else if( flic->var_type == WINDSPEED )
					a12 = sqrt(u*u + v*v);
			}
			else {
				a12 = bad_val;
			}
			wdir = flic->data[k][j][i+1][0] * 0.1;
			wspd = flic->data[k][j][i+1][1] * 0.1;
			x = (i+1) * flic->head->sxt;
			y = j * flic->head->syt;
			if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1)
			{
				if( flic->var_type == RADWIND )a21 = vr;
				else if( flic->var_type == TANWIND )a21 = vt;
				else if( flic->var_type == U_COMPONENT )
					a21 = u;
				else if( flic->var_type == V_COMPONENT )
					a21 = v;
				else if( flic->var_type == WINDSPEED )
					a21 = sqrt(u*u + v*v);
			}
			else {
				a21 = bad_val;
			}
			wdir = flic->data[k][j+1][i+1][0] * 0.1;
			wspd = flic->data[k][j+1][i+1][1] * 0.1;
			x = (i+1) * flic->head->sxt;
			y = (j+1) * flic->head->syt;
			if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1)
			{
				if( flic->var_type == RADWIND )a22 = vr;
				else if( flic->var_type == TANWIND )a22 = vt;
				else if( flic->var_type == U_COMPONENT )
					a22 = u;
				else if( flic->var_type == V_COMPONENT )
					a22 = v;
				else if( flic->var_type == WINDSPEED )
					a22 = sqrt(u*u + v*v);
			}
			else {
				a22 = bad_val;
			}

		}
		zoom_and_paint(flic->xi, i, j, flic->pic_left, flic->pic_bottom,
		        flic->height, bad_val, 
			a12,a11,a22,a21, flic->nzx, flic->nzy,
			zero_contour, zero_color, zscale, zmin);
	     }
	   }
	}
	else if ( flic->orientation == XZ_SLICE) { /* XZ slices */
	  flic->nzx = flic->pic_wid / flic->idim ;
/* Typically the lowest layer in a Doppler analysis is above the ground.
 * If so set nzy smaller, to hold that blank strip at the bottom...
 */
	  if( zc > 0.0 ) {
  		flic->nzy = flic->pic_ht / (1 + flic->kdim ) ;
		kcorn = 1;
	  }
	  else {
	  	flic->nzy = flic->pic_ht / flic->kdim;
	  	kcorn = 0;
	  }

	  j = flic->level;
	  for(k = 0; k < flic->kdim-1; k++ ) {
	    for (i = 0; i < flic->idim-1; i++) {
		if( (flic->var_type == REFLECTIVITY) || 
	    	    (flic->var_type == VERTWINDSPEED) ) {
			a12 = flic->data[k+1][j][i][iv] * val_scaling;
			a11 = flic->data[k][j][i][iv] * val_scaling;
			a22 = flic->data[k+1][j][i+1][iv] * val_scaling;
			a21 = flic->data[k][j][i+1][iv] * val_scaling;
		}
		else {	/* convert wd, ws to u, v and then to Vtan, Vrad */
			/* i know following repetition is ugly, but will
			 * make a function after i get working */
			wdir = flic->data[k][j][i][0] * 0.1;
			wspd = flic->data[k][j][i][1] * 0.1;
			x = i * flic->head->sxt;
			y = j * flic->head->syt;
			if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1)
			{
				if( flic->var_type == RADWIND )a11 = vr;
				else if( flic->var_type == TANWIND )a11 = vt;
				else if( flic->var_type == U_COMPONENT )
					a11 = u;
				else if( flic->var_type == V_COMPONENT )
					a11 = v;
				else if( flic->var_type == WINDSPEED )
					a11 = sqrt(u*u + v*v);
			}
			else {
				a11 = bad_val;
			}
			wdir = flic->data[k+1][j][i][0] * 0.1;
			wspd = flic->data[k+1][j][i][1] * 0.1;
			x = i * flic->head->sxt;
			y = j * flic->head->syt;
			if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1)
			{
				if( flic->var_type == RADWIND )a12 = vr;
				else if( flic->var_type == TANWIND )a12 = vt;
				else if( flic->var_type == U_COMPONENT )
					a12 = u;
				else if( flic->var_type == V_COMPONENT )
					a12 = v;
				else if( flic->var_type == WINDSPEED )
					a12 = sqrt(u*u + v*v);
			}
			else {
				a12 = bad_val;
			}
			wdir = flic->data[k][j][i+1][0] * 0.1;
			wspd = flic->data[k][j][i+1][1] * 0.1;
			x = (i+1) * flic->head->sxt;
			y = j * flic->head->syt;
			if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1)
			{
				if( flic->var_type == RADWIND )a21 = vr;
				else if( flic->var_type == TANWIND )a21 = vt;
				else if( flic->var_type == U_COMPONENT )
					a21 = u;
				else if( flic->var_type == V_COMPONENT )
					a21 = v;
				else if( flic->var_type == WINDSPEED )
					a21 = sqrt(u*u + v*v);
			}
			else {
				a21 = bad_val;
			}
			wdir = flic->data[k+1][j][i+1][0] * 0.1;
			wspd = flic->data[k+1][j][i+1][1] * 0.1;
			x = (i+1) * flic->head->sxt;
			y = j * flic->head->syt;
			if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1)
			{
				if( flic->var_type == RADWIND )a22 = vr;
				else if( flic->var_type == TANWIND )a22 = vt;
				else if( flic->var_type == U_COMPONENT )
					a22 = u;
				else if( flic->var_type == V_COMPONENT )
					a22 = v;
				else if( flic->var_type == WINDSPEED )
					a22 = sqrt(u*u + v*v);
			}
			else {
				a22 = bad_val;
			}

		}
		zoom_and_paint(flic->xi, i, k + kcorn, flic->pic_left, 
		        flic->pic_bottom,
		        flic->height,  bad_val, 
			a12,a11,a22,a21, flic->nzx, flic->nzy,
			zero_contour, zero_color, zscale, zmin);
	     }
	   }
	}
	else if ( flic->orientation == YZ_SLICE) { /* YZ slices */
	  flic->nzx = flic->pic_wid / flic->jdim ;
/* Typically the lowest layer in a Doppler analysis is above the ground.
 * If so set nzy smaller, to hold that blank strip at the bottom...
 */
	  if( zc > 0.0 ) {
  		flic->nzy = flic->pic_ht / (1 + flic->kdim ) ;
		kcorn = 1;
	  }
	  else {
	  	flic->nzy = flic->pic_ht / flic->kdim;
	  	kcorn = 0;
	  }

	  i = flic->level;
	  for(k = 0; k < flic->kdim-1; k++ ) {
	    for (j = 0; j < flic->jdim-1; j++) {
		if( (flic->var_type == REFLECTIVITY) || 
	    	    (flic->var_type == VERTWINDSPEED)) {
			a12 = flic->data[k+1][j][i][iv] * val_scaling;
			a11 = flic->data[k][j][i][iv] * val_scaling;
			a22 = flic->data[k+1][j+1][i][iv] * val_scaling;
			a21 = flic->data[k][j+1][i][iv] * val_scaling;
		}
		else {	/* convert wd, ws to u, v and then to Vtan, Vrad */
			/* i know following repetition is ugly, but will
			 * make a function after i get working */
			wdir = flic->data[k][j][i][0] * 0.1;
			wspd = flic->data[k][j][i][1] * 0.1;
			x = i * flic->head->sxt;
			y = j * flic->head->syt;
			if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1)
			{
				if( flic->var_type == RADWIND )a11 = vr;
				else if( flic->var_type == TANWIND )a11 = vt;
				else if( flic->var_type == U_COMPONENT )
					a11 = u;
				else if( flic->var_type == V_COMPONENT )
					a11 = v;
				else if( flic->var_type == WINDSPEED )
					a11 = sqrt(u*u + v*v);
			}
			else {
				a11 = bad_val;
			}
			wdir = flic->data[k+1][j][i][0] * 0.1;
			wspd = flic->data[k+1][j][i][1] * 0.1;
			x = i * flic->head->sxt;
			y = j * flic->head->syt;
			if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1)
			{
				if( flic->var_type == RADWIND )a12 = vr;
				else if( flic->var_type == TANWIND )a12 = vt;
				else if( flic->var_type == U_COMPONENT )
					a12 = u;
				else if( flic->var_type == V_COMPONENT )
					a12 = v;
				else if( flic->var_type == WINDSPEED )
					a12 = sqrt(u*u + v*v);
			}
			else {
				a12 = bad_val;
			}
			wdir = flic->data[k][j+1][i][0] * 0.1;
			wspd = flic->data[k][j+1][i][1] * 0.1;
			x = i * flic->head->sxt;
			y = (j+1) * flic->head->syt;
			if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1)
			{
				if( flic->var_type == RADWIND )a21 = vr;
				else if( flic->var_type == TANWIND )a21 = vt;
				else if( flic->var_type == U_COMPONENT )
					a21 = u;
				else if( flic->var_type == V_COMPONENT )
					a21 = v;
				else if( flic->var_type == WINDSPEED )
					a21 = sqrt(u*u + v*v);
			}
			else {
				a21 = bad_val;
			}
			wdir = flic->data[k+1][j+1][i][0] * 0.1;
			wspd = flic->data[k+1][j+1][i][1] * 0.1;
			x = i * flic->head->sxt;
			y = (j+1) * flic->head->syt;
			if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1)
			{
				if( flic->var_type == RADWIND )a22 = vr;
				else if( flic->var_type == TANWIND )a22 = vt;
				else if( flic->var_type == U_COMPONENT )
					a22 = u;
				else if( flic->var_type == V_COMPONENT )
					a22 = v;
				else if( flic->var_type == WINDSPEED )
					a22 = sqrt(u*u + v*v);
			}
			else {
				a22 = bad_val;
			}

		}
		zoom_and_paint(flic->xi, j, k + kcorn, flic->pic_left, flic->pic_bottom,
		        flic->height, bad_val, 
			a12,a11,a22,a21, flic->nzx, flic->nzy,
			zero_contour, zero_color, zscale, zmin);
	     }
	   }
	}
	
	else if ( flic->orientation == RZ_SLICE) { /* RZ slices */
/* 
 *	Note that in the radial average dBZ are converted to Z, averaged,
 * and then converted back to dBZ. The interpolation for the color display,
 * however, is done in DBZ. Changed on 24 Sept 1996.
 */
  /* First, compute the radial average */
	  for(k = 0; k < flic->kdim; k++ ) {
	    	for (i = 0; i < RMAX; i++) {
	    		rav_array[k][i][0] = 0.0;
	    		rav_array[k][i][1] = 0.0;
		}
	  }
	  d_rav = (RMAX-1) / (flic->rad_max - flic->rad_min);
	  for(k = 0; k < flic->kdim; k++ ) {
	    for (j = 0; j < flic->jdim; j++) {
		y = j * flic->head->syt;
		if( flic->shift_center )
			ysq = (y - yc + flic->cen_shift[1]) * 
				(y - yc + flic->cen_shift[1]);
		else
			ysq = (y-yc) * (y-yc);
	    	for (i = 0; i < flic->idim; i++) {
		    x = i * flic->head->sxt;
		    if( flic->shift_center )
		    	radius = (x - xc + flic->cen_shift[0]) * 
		    		(x - xc + flic->cen_shift[0]) + ysq;
		    else
		    	radius = (x - xc) * (x - xc) + ysq;
		    radius = sqrt( radius);
		    ir = (radius - flic->rad_min) * d_rav;
		    if((ir < RMAX) && !(ir < 0) ) { /* -------------------- */
			if(flic->var_type == REFLECTIVITY) {
				a11 = flic->data[k][j][i][iv] * val_scaling;
				if( a11 > bad_val ) { /* dBZ to Z */
					a11 = a11 * 0.1;
					a11 = pow(10.0, a11);
				}
			}
	    	    	else if (flic->var_type == VERTWINDSPEED) {
				a11 = flic->data[k][j][i][iv] * val_scaling;
			}
			else {
			   wdir = flic->data[k][j][i][0] * 0.1;
			   wspd = flic->data[k][j][i][1] * 0.1;
			   if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1) {
				   if( flic->var_type == RADWIND )a11 = vr;
				   else if( flic->var_type == TANWIND )a11 = vt;
				   else if( flic->var_type == U_COMPONENT )
					a11 = u;
				   else if( flic->var_type == V_COMPONENT )
					a11 = v;
				   else if( flic->var_type == WINDSPEED )
					a11 = sqrt(u*u + v*v);
			   }
			   else 
				a11 = bad_val;
			}
			if( a11 > bad_val ) {
				rav_array[k][ir][0] =
				  rav_array[k][ir][0] + a11;
				rav_array[k][ir][1] =
				  rav_array[k][ir][1] + 1.0;
			}
		    } /* ----------------- */
	    	}
	    }
	   }
	/* then go through and interpolate for display */
	  flic->nzx = flic->pic_wid / RMAX ; /* RMAX is dim of radial avg array */
/* Typically the lowest layer in a Doppler analysis is above the ground.
 * If so set nzy smaller, to hold that blank strip at the bottom...
 */
	  if( zc > 0.0 ) {
  		flic->nzy = flic->pic_ht / (1 + flic->kdim ) ;
		kcorn = 1;
	  }
	  else {
	  	flic->nzy = flic->pic_ht / flic->kdim;
	  	kcorn = 0;
	  }
/* Note to display dBZ we have to convert the average Z back to dBZ...
 */
	   for( k=0; k < flic->kdim-1; k++) {
	   	for(ir=0; ir < RMAX-1; ir++) {
	   		if( rav_array[k][ir][1] > 0.0 ) {
	   		   a11 = rav_array[k][ir][0] / rav_array[k][ir][1];
			   if(flic->var_type == REFLECTIVITY) 
					a11 = 10.0 * log10(a11);
			}
	   		else
	   		   a11 = bad_val;
	   		   
	   		if( rav_array[k+1][ir][1] > 0.0 ) {
	   		   a12 = rav_array[k+1][ir][0] / rav_array[k+1][ir][1];
			   if(flic->var_type == REFLECTIVITY) 
					a12 = 10.0 * log10(a12);
			}
	   		else
	   		   a12 = bad_val;
	   		   
	   		if( rav_array[k+1][ir+1][1] > 0.0 ) {
	   		   a22 = rav_array[k+1][ir+1][0] / 
	   		   	rav_array[k+1][ir+1][1];
			   if(flic->var_type == REFLECTIVITY) 
					a22 = 10.0 * log10(a22);
			}
	   		else
	   		   a22 = bad_val;
	   		   
	   		if( rav_array[k][ir+1][1] > 0.0 ) {
	   		   a21 = rav_array[k][ir+1][0] / rav_array[k][ir+1][1];
			   if(flic->var_type == REFLECTIVITY) 
					a21 = 10.0 * log10(a21);
			}
	   		else
	   		   a21 = bad_val;
		
			zoom_and_paint(flic->xi, ir, k + kcorn,
				flic->pic_left, flic->pic_bottom, 
				flic->height, bad_val, a12, a11, a22, a21, 
				flic->nzx, flic->nzy,
				zero_contour, zero_color, zscale, zmin);
	   	}
	   }
	} /* RZ mean block */

/* Added azimuth-height display 4 June 1997 */
	else if ( flic->orientation == AZ_SLICE) { /* Az - Z slices */
/* First, compile the azimuthal slice */
	  for(k = 0; k < flic->kdim; k++ ) {
	    	for (i = 0; i < RMAX; i++) {
	    		rav_array[k][i][0] = 0.0;
	    		rav_array[k][i][1] = 0.0;
		}
	  }
	  d_az = (flic->az_max - flic->az_min) / (RMAX-1);
	  radius = flic->level;
	  for (ia = 0; ia < RMAX; ia++) {
	    az = flic->az_min + ia * d_az;
	    x = xc + radius * cos(az);
	    y = yc + radius * sin(az);
	    if( flic->shift_center ) {
		y = y + flic->cen_shift[1];
		x = x + flic->cen_shift[0];
	    }
	    i = x / flic->head->sxt;
	    j = y / flic->head->syt;
	    if( i>=0 && i<flic->idim && j>=0 && j<flic->jdim) {
		for(k = 0; k < flic->kdim; k++ ) {
			if(flic->var_type == REFLECTIVITY) {
				a11 = flic->data[k][j][i][iv] * val_scaling;
				if( a11 > bad_val ) { /* dBZ to Z */
					a11 = a11 * 0.1;
					a11 = pow(10.0, a11);
				}
			}
	    	    	else if (flic->var_type == VERTWINDSPEED) {
				a11 = flic->data[k][j][i][iv] * val_scaling;
			}
			else {
			   wdir = flic->data[k][j][i][0] * 0.1;
			   wspd = flic->data[k][j][i][1] * 0.1;
			   if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1) {
				   if( flic->var_type == RADWIND )a11 = vr;
				   else if( flic->var_type == TANWIND )a11 = vt;
				   else if( flic->var_type == U_COMPONENT )
					a11 = u;
				   else if( flic->var_type == V_COMPONENT )
					a11 = v;
				   else if( flic->var_type == WINDSPEED )
					a11 = sqrt(u*u + v*v);
			   }
			   else 
				a11 = bad_val;
			}
			if( a11 > bad_val ) {
				rav_array[k][RMAX-1-ia][0] =
				  rav_array[k][RMAX-1-ia][0] + a11;
				rav_array[k][RMAX-1-ia][1] =
				  rav_array[k][RMAX-1-ia][1] + 1.0;
			}
		} /* end k loop, data in bounds */
	    }
	   } /* end ia loop */
	/* then go through and interpolate for display */
	  flic->nzx = flic->pic_wid / RMAX ; 
/* Typically the lowest layer in a Doppler analysis is above the ground.
 * If so set nzy smaller, to hold that blank strip at the bottom...
 */
	  if( zc > 0.0 ) {
  		flic->nzy = flic->pic_ht / (1 + flic->kdim ) ;
		kcorn = 1;
	  }
	  else {
	  	flic->nzy = flic->pic_ht / flic->kdim;
	  	kcorn = 0;
	  }
/* Note to display dBZ we have to convert the average Z back to dBZ...
 */
	   for( k=0; k < flic->kdim-1; k++) {
	   	for(ir=0; ir < RMAX-1; ir++) {
	   		if( rav_array[k][ir][1] > 0.0 ) {
	   		   a11 = rav_array[k][ir][0] / rav_array[k][ir][1];
			   if(flic->var_type == REFLECTIVITY) 
					a11 = 10.0 * log10(a11);
			}
	   		else
	   		   a11 = bad_val;
	   		   
	   		if( rav_array[k+1][ir][1] > 0.0 ) {
	   		   a12 = rav_array[k+1][ir][0] / rav_array[k+1][ir][1];
			   if(flic->var_type == REFLECTIVITY) 
					a12 = 10.0 * log10(a12);
			}
	   		else
	   		   a12 = bad_val;
	   		   
	   		if( rav_array[k+1][ir+1][1] > 0.0 ) {
	   		   a22 = rav_array[k+1][ir+1][0] / 
	   		   	rav_array[k+1][ir+1][1];
			   if(flic->var_type == REFLECTIVITY) 
					a22 = 10.0 * log10(a22);
			}
	   		else
	   		   a22 = bad_val;
	   		   
	   		if( rav_array[k][ir+1][1] > 0.0 ) {
	   		   a21 = rav_array[k][ir+1][0] / rav_array[k][ir+1][1];
			   if(flic->var_type == REFLECTIVITY) 
					a21 = 10.0 * log10(a21);
			}
	   		else
	   		   a21 = bad_val;
		
			zoom_and_paint(flic->xi, ir, k + kcorn,
				flic->pic_left, flic->pic_bottom, 
				flic->height, bad_val, a12, a11, a22, a21, 
				flic->nzx, flic->nzy,
				zero_contour, zero_color, zscale, zmin);
	   	}
	   }
	} /* AZ - height  mean block */

/* Added arbitrary slice display 18 August 1997 */
	else if ( flic->orientation == ARB_SLICE) { /* Arbitrary slice */
/* First, compile the slice */
	  for(k = 0; k < flic->kdim; k++ ) {
	    	for (i = 0; i < RMAX; i++) {
	    		rav_array[k][i][0] = 0.0;
	    		rav_array[k][i][1] = 0.0;
		}
	  }
	  radius = sqrt( (flic->x_end - flic->x_beg)*(flic->x_end - flic->x_beg) +
	  	(flic->y_end - flic->y_beg)*(flic->y_end - flic->y_beg) );
	  d_s = radius / (RMAX-1);
	  az = atan2((flic->y_end - flic->y_beg),
	  	(flic->x_end - flic->x_beg));
	  for (ia = 0; ia < RMAX; ia++) {
	    x = flic->x_beg + ia * d_s * cos(az);
	    y = flic->y_beg + ia * d_s * sin(az);
	    i = floor(x);
	    if( fabs(i-x) > 0.5) i = ceil(x);
	    j = floor(y);
	    if( fabs( j - y) > 0.5) j = ceil(y);
/* DEBUG
	    printf("ia: %d x,y: (%.1f, %.1f), i,j: %d, %d \n",
	    	ia, x, y, i, j);
 */
	    if( i>=0 && i<flic->idim && j>=0 && j<flic->jdim) {
		for(k = 0; k < flic->kdim; k++ ) {
			if(flic->var_type == REFLECTIVITY) {
				a11 = flic->data[k][j][i][iv] * val_scaling;
				if( a11 > bad_val ) { /* dBZ to Z */
					a11 = a11 * 0.1;
					a11 = pow(10.0, a11);
				}
			}
	    	    	else if (flic->var_type == VERTWINDSPEED) {
				a11 = flic->data[k][j][i][iv] * val_scaling;
			}
			else {
			   wdir = flic->data[k][j][i][0] * 0.1;
			   wspd = flic->data[k][j][i][1] * 0.1;
			   if(wdws_to_vtvr(wdir, wspd, x, y, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1) {
				   if( flic->var_type == RADWIND )a11 = vr;
				   else if( flic->var_type == TANWIND )a11 = vt;
				   else if( flic->var_type == U_COMPONENT )
					a11 = u;
				   else if( flic->var_type == V_COMPONENT )
					a11 = v;
				   else if( flic->var_type == WINDSPEED )
					a11 = sqrt(u*u + v*v);
			   }
			   else 
				a11 = bad_val;
			}
			if( a11 > bad_val ) {
				rav_array[k][ia][0] = a11;
				  rav_array[k][ia][1] = 1.0;
			}
		} /* end k loop, data in bounds */
	    }
	   } /* end ia loop */
	/* then go through and interpolate for display */
	  flic->nzx = flic->pic_wid / RMAX ; 
/* Typically the lowest layer in a Doppler analysis is above the ground.
 * If so set nzy smaller, to hold that blank strip at the bottom...
 */
	  if( zc > 0.0 ) {
  		flic->nzy = flic->pic_ht / (1 + flic->kdim ) ;
		kcorn = 1;
	  }
	  else {
	  	flic->nzy = flic->pic_ht / flic->kdim;
	  	kcorn = 0;
	  }
/* Note to display dBZ we have to convert the average Z back to dBZ...
 */
	   for( k=0; k < flic->kdim-1; k++) {
	   	for(ir=0; ir < RMAX-1; ir++) {
	   		if( rav_array[k][ir][1] > 0.0 ) {
	   		   a11 = rav_array[k][ir][0];
			   if(flic->var_type == REFLECTIVITY) 
					a11 = 10.0 * log10(a11);
			}
	   		else
	   		   a11 = bad_val;
	   		   
	   		if( rav_array[k+1][ir][1] > 0.0 ) {
	   		   a12 = rav_array[k+1][ir][0];
			   if(flic->var_type == REFLECTIVITY) 
					a12 = 10.0 * log10(a12);
			}
	   		else
	   		   a12 = bad_val;
	   		   
	   		if( rav_array[k+1][ir+1][1] > 0.0 ) {
	   		   a22 = rav_array[k+1][ir+1][0];
			   if(flic->var_type == REFLECTIVITY) 
					a22 = 10.0 * log10(a22);
			}
	   		else
	   		   a22 = bad_val;
	   		   
	   		if( rav_array[k][ir+1][1] > 0.0 ) {
	   		   a21 = rav_array[k][ir+1][0];
			   if(flic->var_type == REFLECTIVITY) 
					a21 = 10.0 * log10(a21);
			}
	   		else
	   		   a21 = bad_val;
		
			zoom_and_paint(flic->xi, ir, k + kcorn,
				flic->pic_left, flic->pic_bottom, 
				flic->height, bad_val, a12, a11, a22, a21, 
				flic->nzx, flic->nzy,
				zero_contour, zero_color, zscale, zmin);
	   	}
	   }
	} /* Arbitrary slice block */

/* Copy XImage into pixmap */
	XPutImage(XtDisplay(flic->widget), pixmap, gc, flic->xi, 
		0, 0, 0, 0, flic->width, flic->height);

/* Add caption and grid */
	ColorScale(flic->widget, flic, call_data);
	set_color(flic->widget, "Black");
	Grid(flic->widget, flic, call_data);
	Caption(flic->widget, flic, call_data);

/* render pixmap into window */
	XCopyArea(XtDisplay(flic->widget), pixmap, XtWindow(flic->widget),
		gc, 0, 0, flic->width, flic->height, 0, 0);

/* reset color for subsequent drawing
 * set White so arrows will show up over color */
	set_color(flic->widget, "White");
}
  
void zoom_and_paint(xi, i, j, pic_left, pic_bottom, ht, 
	bad_val, a12,a11,a22,a21, nzx, nzy,
	zero_contour, zero_color, zscale, zmin)
XImage *xi;
int i, j, ht, pic_left, pic_bottom;
float bad_val,a12, a11, a22, a21;
int nzx, nzy, zero_contour, zero_color;
float zscale, zmin;
/* function to zoom by integral interpolation, adapted from f77 zoom_int.f
 * note that i pass ht because X assumes top of window is 0,0, while
 * my radar data arrays are created with 0,0 being lower left corner,
 * so jscreen = ht - jarray(interpolated) for XPutPixel.
 */
/* col_pix is GLOBAL array of color pixels corresponding to colors */
{
	float divert, dihorz, gauche, droit, av, deleft, deright;
	float top, bottom, dtop, dbot, delvert, delhor, zint;
	int l, k, jcorn, icorn, nm, col_num;
	unsigned long pixie;
	
	divert=1.0/nzy;
	dihorz=1.0/nzx;
/* Lower left corner: */
	icorn = pic_left + i * nzx;
	jcorn = pic_bottom + j * nzy;
	
	av = 0.0;
	nm = 0;
	if( a12 > bad_val ) {
		nm = nm + 1;
		av = av + a12;
	}
	if( a11 > bad_val ) {
		nm = nm + 1;
		av = av + a11;
	}
	if( a22 > bad_val ) {
		nm = nm + 1;
		av = av + a22;
	}
	if( a21 > bad_val ) {
		nm = nm + 1;
		av = av + a21;
	}

	if(nm >= 1) {  /* do interpolation */
		if(nm < 4) { /* use average at bad points */
			av = av/nm;
			if(a11 <= bad_val) a11 = av;
			if(a12 <= bad_val) a12 = av;
			if(a21 <= bad_val) a21 = av;
			if(a22 <= bad_val) a22 = av;
		}
		deleft = ( a12 - a11 ) * divert;
		deright = ( a22 - a21 ) * divert;
		gauche = a11;
		droit = a21;
		for ( k=0; k< nzy; k++) {
		  delhor=(droit-gauche)*dihorz;
		  zint=gauche ;
		  for ( l=0; l<nzx; l++){
		     if( zint > zmin ) {
			col_num = (zint - zmin) * zscale ;
			pixie = col_pix[col_num];
			XPutPixel(xi, icorn + l, ht - (jcorn + k), pixie);
		     }
		     zint = zint + delhor;
		  }
		  if( zero_contour == 1) { /* check for zero */
		   if( gauche*droit <= 0.0 ) { /* zero somewhere */
		     if( gauche == droit ){ /* zero line is horizontal */
			for( l=0; l < nzx; l++) { /* set zero */
			pixie = col_pix[zero_color];
			XPutPixel(xi, icorn + l, ht - (jcorn + k), pixie);
			}
		     }
		     else { /* find pixel nearest zero (this is buggy) */
			l = floor( -gauche / delhor);
			pixie = col_pix[zero_color];
			XPutPixel(xi, icorn + l, ht - (jcorn + k), pixie);
 		     }
		   }
	         }
		 gauche=gauche+deleft;
		 droit=droit+deright;
		}

	      if( zero_contour == 1 ) {   /* have to look in other dir too */
		top = a12;
		bottom = a11;
		dtop = (a22 - a12) * dihorz;
		dbot = (a21 - a11) * dihorz;
		delvert = (top-bottom) * divert;
		for( l=0; l< nzx; l++) {
		   if( top*bottom <= 0.0 ) {
		     if( top == bottom ){
			for ( k=0; k<nzy; k++) {
			pixie = col_pix[zero_color];
			XPutPixel(xi, icorn + l, ht - (jcorn + k), pixie);
			}
		     }
		     else {
			k = floor( -bottom / delvert);
			pixie = col_pix[zero_color];
			XPutPixel(xi, icorn + l, ht - (jcorn + k), pixie);
		     }
		     top = top + dtop;
		     bottom = bottom + dbot;
		   }
		}
	      }
	}

	else { 		/* fill with zero */
		pixie = col_pix[zero_color];
		for ( k=0; k< nzy; k++) {
			for (l = 0; l<nzx; l++) {
			    XPutPixel(xi, icorn + l, ht -(jcorn + k), pixie);
			}
		}
	}
}


/* ARGSUSED */	
void WindOverlay(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;

{
	/* cast required in ANSI C , but I could not get to work if
	 * cast the way they did in O'Reilly....*/
	struct chitra *flic = client_data;
	void Grid(), Caption();
	
	int i, j, k, kcorn, iv, iwd, iht;
	int nskip, ns_j, i_start, ir, ia;
	Position x, y;
	double shaft, shaft_max, speed_max, wvert, bad_w;
	float wdir, wspd, u, v, vt, vr, bad_val, radius, d_rav, 
		xp, yp, xc, yc, ysq, az, d_s;
	float frac;
	float rav_array[RMAX][4];
	char string[8];
	int len;	
	bad_val = -900.0;
	bad_w = -90.0;
/*        printf("WindOverlay is being called \n"); */
/* 9 June 1995... Modified sections that plot vertical slices and R-Z means
 * so that the magnitude of arrows is scaled the same in horizontal
 * and vertical, but direction of arrows is scaled by vertical exaggeration
 * factor. Thus eye can join up arrows for streamlines / trajectories
 *	1 November 1995... changed so that arrow shaft length in vertical 
 * slices corresponds to vertical exaggeration. Also changed horizontal plots
 * to arrows instead of wind barbs. I will make this an option eventually.
 * Also, require that some fraction of points (flic.arc_min_frac) in ring have good data
 * to display RZ mean plots.
 *	20 November 1995...changed so that arrows are plotted at correct level
 * in vertical slices: first level at height zz etc...
 *	25 February 1997 ... added code to set line widths of arrows,
 * and to use info in flic to set shaft length. */
 
/* speed_max is for scaling arrows on cross sections...i should make
 * this user adjustable !
 */

/* Set line widths for arrows, and sharp corners */
/*        printf("WindOverlays 6 \n"); */
	XSetLineAttributes(XtDisplay(flic->widget), gc, 
		flic->arrow_shaft_width, LineSolid, CapRound, JoinMiter);

/* XY slices, so draw wind barbs or arrows */	
/*        printf("Wind Overlay 5 \n"); */
	if ( flic->orientation == XY_SLICE) { 
	  flic->nzx = flic->pic_wid / flic->idim ;
	  flic->nzy = flic->pic_ht / flic->jdim ;
	  if( flic->nzx < flic->nzy) flic->nzy = flic->nzx; /* square up */
	  flic->nzx = flic->nzy;
	  k = flic->level;
	  nskip = flic->arrow_interval;
/*          printf("WindOverlay 1 \n"); */
	  if( flic->xy_arrows) {
		  speed_max = 50.0;
		  shaft_max = flic->arrow_shaft_length / speed_max;
	  }
	  else
		  shaft = flic->arrow_shaft_length;
/* If nskip = 2, then stagger rows */
	  if( nskip == 2) ns_j = 1;
	  else ns_j = nskip;
	  for (j = 0; j < flic->jdim; j+=ns_j ) {
	    if( nskip ==2 ) i_start = j&1;
	    else i_start = 0;
	    for (i = i_start; i < flic->idim; i+=nskip) {
		wdir = flic->data[k][j][i][0] * 0.1;
		wspd = flic->data[k][j][i][1] * 0.1;
		if( wdir2uv(wdir, wspd, &u, &v, bad_val) != -1){
		   if( flic->remove_motion) {
			u = u - flic->storm_uv[0];
			v = v - flic->storm_uv[1];
			if( uv2wd(u, v, &wdir, &wspd, bad_val) == -1)
					wspd = bad_val;
		   }
		   x = flic->pic_left + i * flic->nzx;
		   y = flic->height - ( flic->pic_bottom + j * flic->nzy);
		   if( flic->xy_arrows ) {
		    shaft = shaft_max * sqrt( u*u + v*v );
		    xplot_arrow(XtDisplay(flic->widget), XtWindow(flic->widget),
			gc, x, y, shaft, u, v);
		    /* also to (global) pixmap, so will scroll */
		    xplot_arrow(XtDisplay(flic->widget), pixmap,
			gc, x, y, shaft, u, v );
		   }
		   else {	/* plot wind barbs */
		      xplot_barb_wd(XtDisplay(flic->widget), XtWindow(flic->widget),
			gc, x, y, wdir, wspd, shaft);
		      /* also to (global) pixmap, so will scroll */
		      xplot_barb_wd(XtDisplay(flic->widget), pixmap,
			gc, x, y, wdir, wspd, shaft);
		   }
		} /* good wind block */
	     } /* i loop */
	   } /* j loop */
/*           printf("WindOverlay 2 \n"); */
	   if( flic->xy_arrows ) {
/* dealy-bobber to show scaling of arrows: */
	      x = flic->pic_left + flic->pic_wid + 10.0;
	      y = flic->height - (  flic->pic_bottom + flic->pic_ht + 10.0);
	      shaft = speed_max * shaft_max;
	      xplot_arrow(XtDisplay(flic->widget), 
	  		XtWindow(flic->widget), gc, x, y, shaft, 0.0, 
	  		speed_max);
	      xplot_arrow(XtDisplay(flic->widget), 
	  		pixmap, gc, x, y, shaft, 0.0, 
	  		speed_max );
	      xplot_arrow(XtDisplay(flic->widget), 
	  		XtWindow(flic->widget), gc, x, y, shaft, 
	  		speed_max, 0.0);
	      xplot_arrow(XtDisplay(flic->widget), 
	  		pixmap, gc, x, y, shaft, speed_max, 0.0);
	      len = sprintf(string,"%.1f m/s\n", speed_max);
/*              printf("WindowOverlay after len \n"); */
	      XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x+5, y-5, string, len);
	      XDrawString(XtDisplay(flic->widget), XtWindow(flic->widget),
			gc, x+5, y-5, string, len);
/*              printf("WindowOverlay after len \n"); */
	   }
/* Set back to Black, as default */
	   set_color(flic->widget, "Black");
	}
	
/* XZ slices, so do arrows */
	else if ( flic->orientation == XZ_SLICE) { 
	  flic->nzx = flic->pic_wid / flic->idim ; /* pixels per array element */
/* Typically the lowest layer in a Doppler analysis is above the ground.
 * If so set nzy smaller, to hold that blank strip at the bottom...
 */
	  if( flic->head->zz > 0.0 ) {
  		flic->nzy = flic->pic_ht / (1 + flic->kdim ) ;
		kcorn = 1;
	  }
	  else {
	  	flic->nzy = flic->pic_ht / flic->kdim;
	  	kcorn = 0;
	  }

	  flic->w_exag = (flic->head->sxt / flic->nzx) / 
	  	(flic->head->szt / flic->nzy);
	  j = flic->level;
	  nskip = flic->arrow_interval;
	  speed_max = 10.0;
	  shaft_max = flic->arrow_shaft_length / speed_max;

	  for(k = 0; k < flic->kdim; k++ ) {
	    for (i = 0; i < flic->idim; i++) {
		wdir = flic->data[k][j][i][0] * 0.1;
		wspd = flic->data[k][j][i][1] * 0.1;
		wvert = flic->data[k][j][i][2] * 0.01;
		if( wvert > bad_w){
		  if(wdir2uv(wdir, wspd, &u, &v, bad_val) != -1){
		    if( flic->remove_motion)
		    	u = u - flic->storm_uv[0];
		    x = flic->pic_left + i * flic->nzx;
		    y = flic->height - (  flic->pic_bottom + 
		    	(k + kcorn) * flic->nzy);
		    shaft = shaft_max * sqrt(u*u + wvert * flic->w_exag 
		    	* wvert  * flic->w_exag);
		    xplot_arrow(XtDisplay(flic->widget), XtWindow(flic->widget),
			gc, x, y, shaft, u, wvert *flic->w_exag);
		    /* also to (global) pixmap, so will scroll */
		    xplot_arrow(XtDisplay(flic->widget), pixmap,
			gc, x, y, shaft, u, wvert * flic->w_exag );

		  }
	        }
	     }
	   }
/* Set back to Black, as default */
	set_color(flic->widget, "Black");
/* dealy-bobber to show scaling of arrows: */
	   x = flic->pic_left + flic->pic_wid + 10.0;
	   y = flic->height - (  flic->pic_bottom + flic->pic_ht + 10.0);
	   shaft = 5.0 * shaft_max  * flic->w_exag;
	   xplot_arrow(XtDisplay(flic->widget), 
	  		XtWindow(flic->widget), gc, x, y, shaft, 0.0, 
	  		5.0  * flic->w_exag );
	   xplot_arrow(XtDisplay(flic->widget), 
	  		pixmap, gc, x, y, shaft, 0.0, 
	  		5.0 * flic->w_exag );
	   shaft = 5.0 * shaft_max;
	   xplot_arrow(XtDisplay(flic->widget), 
	  		XtWindow(flic->widget), gc, x, y, shaft, 5.0, 0.0);
	   xplot_arrow(XtDisplay(flic->widget), 
	  		pixmap, gc, x, y, shaft, 5.0, 0.0);
	   len = sprintf(string,"5 m/s\n");
	   XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x+5, y-5, string, len);
	   XDrawString(XtDisplay(flic->widget), XtWindow(flic->widget),
			gc, x+5, y-5, string, len);

	}
	
/* YZ slices */
	else if ( flic->orientation == YZ_SLICE) { 
	  flic->nzx = flic->pic_wid / flic->jdim ;
/* Typically the lowest layer in a Doppler analysis is above the ground.
 * If so set nzy smaller, to hold that blank strip at the bottom...
 */
	  if( flic->head->zz > 0.0 ) {
  		flic->nzy = flic->pic_ht / (1 + flic->kdim ) ;
		kcorn = 1;
	  }
	  else {
	  	flic->nzy = flic->pic_ht / flic->kdim;
	  	kcorn = 0;
	  }

	  flic->w_exag = ( flic->head->syt / flic->nzx) / 
	  	(flic->head->szt / flic->nzy);
	  i = flic->level;
	  nskip = flic->arrow_interval;
	  speed_max = 10.0;
	  shaft_max = flic->arrow_shaft_length / speed_max;

 	  for (k = 0; k < flic->kdim; k++ ) {
	    for (j = 0; j < flic->jdim; j++) {

		wdir = flic->data[k][j][i][0] * 0.1;
		wspd = flic->data[k][j][i][1] * 0.1;
		wvert = flic->data[k][j][i][2] * 0.01;
		if( wvert > bad_w ) {
		  if(wdir2uv(wdir, wspd, &u, &v, bad_val) != -1){
			if( flic->remove_motion)
		    		v = v - flic->storm_uv[1];
		  	x = flic->pic_left + j * flic->nzx;
		  	y = flic->height - (  flic->pic_bottom + 
		  		(k + kcorn) * flic->nzy);
			shaft = shaft_max * sqrt(v*v + wvert  * flic->w_exag
				* wvert  * flic->w_exag );
		  	xplot_arrow(XtDisplay(flic->widget), 
		  		XtWindow(flic->widget), gc, x, y, shaft, v, 
		  		wvert * flic->w_exag);
			/* also to (global) pixmap, so will scroll */
		  	xplot_arrow(XtDisplay(flic->widget), 
		  		pixmap, gc, x, y, shaft, v, 
		  		wvert * flic->w_exag);
		  }
		}
	     }
	   }
/* Set back to Black, as default */
	set_color(flic->widget, "Black");
/* dealy-bobber to show scaling of arrows: */
	   x = flic->pic_left + flic->pic_wid + 10.0;
	   y = flic->height - (  flic->pic_bottom + flic->pic_ht + 10.0);
	   shaft = 5.0 * shaft_max  * flic->w_exag ;
	   xplot_arrow(XtDisplay(flic->widget), 
	  		XtWindow(flic->widget), gc, x, y, shaft, 0.0, 
	  		5.0 * flic->w_exag );
	   xplot_arrow(XtDisplay(flic->widget), 
	  		pixmap, gc, x, y, shaft, 0.0, 
	  		5.0 * flic->w_exag );
	   shaft = 5.0 * shaft_max;
	   xplot_arrow(XtDisplay(flic->widget), 
	  		XtWindow(flic->widget), gc, x, y, shaft, 5.0, 0.0);
	   xplot_arrow(XtDisplay(flic->widget), 
	  		pixmap, gc, x, y, shaft, 5.0, 0.0);
	   len = sprintf(string,"5 m/s\n");
	   XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x+5, y-5, string, len);
	   XDrawString(XtDisplay(flic->widget), XtWindow(flic->widget),
			gc, x+5, y-5, string, len);

	}
	
/* RZ mean slices ... a little more work */
	else if ( flic->orientation == RZ_SLICE) { 
	  xc = flic->head->xz;
	  yc = flic->head->yz;
	  flic->nzx = flic->pic_wid / RMAX ; /* RMAX is dim of radial avg array */
/* Typically the lowest layer in a Doppler analysis is above the ground.
 * If so set nzy smaller, to hold that blank strip at the bottom...
 */
	  if( flic->head->zz > 0.0 ) {
  		flic->nzy = flic->pic_ht / (1 + flic->kdim ) ;
		kcorn = 1;
	  }
	  else {
	  	flic->nzy = flic->pic_ht / flic->kdim;
	  	kcorn = 0;
	  }
	  speed_max = 10.0;
	  shaft_max = flic->arrow_shaft_length / speed_max;
	  d_rav = (RMAX-1) / (flic->rad_max - flic->rad_min);
	  flic->w_exag = ( (1.0/ d_rav) / flic->nzx) / 
	  	(flic->head->szt / flic->nzy);
	 
 	  for(k = 0; k < flic->kdim; k++ ) {
	    y = flic->height - (  flic->pic_bottom + (k+kcorn) * flic->nzy);

 	    /* Zero out rz mean slice: */
	    for (i = 0; i < RMAX; i++) {
	    		rav_array[i][0] = 0.0;
	    		rav_array[i][1] = 0.0;
	    		rav_array[i][2] = 0.0;
	    		rav_array[i][3] = 0.0;
	    }

	    /* Load rz mean slice with radial and vertical winds */
	    for (j = 0; j < flic->jdim; j++) {
		yp = j * flic->head->syt;
		if( flic->shift_center )
			ysq = (yp-yc + flic->cen_shift[1]) * 
				(yp-yc + flic->cen_shift[1]);
		else
			ysq = (yp-yc) * (yp-yc);
	    	for (i = 0; i < flic->idim; i++) {
		    xp = i * flic->head->sxt;
		    if( flic->shift_center )
			radius = ysq + (xp-xc + flic->cen_shift[0]) * 
				(xp-xc + flic->cen_shift[0]);
		    else
		    	radius = (xp-xc)*(xp-xc) + ysq;
		    radius = sqrt( radius);
		    ir = (radius - flic->rad_min) * d_rav;
		    if( (ir < RMAX ) && !(ir < 0) ) { /* -------------------- */
			rav_array[ir][3] = rav_array[ir][3] + 1.0;
			wdir = flic->data[k][j][i][0] * 0.1;
			wspd = flic->data[k][j][i][1] * 0.1;
			wvert = flic->data[k][j][i][2] * 0.01;
			if ( wvert > bad_w  ){
			 if(wdws_to_vtvr(wdir, wspd, xp, yp, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1) {
			   rav_array[ir][0] = rav_array[ir][0] + 1.0;
			   rav_array[ir][1] = rav_array[ir][1] + vr;
			   rav_array[ir][2] = rav_array[ir][2] + wvert;
			 }
			}
		     } /* ------------------------------------- */
		}
	   }

	   /* now plot the mean winds at this level */
	   for ( ir = 0; ir < RMAX; ir++ ) {
		frac = rav_array[ir][0] / rav_array[ir][3];
	   	if( (rav_array[ir][0] > 0.0) && 
	   	   !(frac < flic->arc_min_frac) ) {
	   		x = flic->pic_left + ir * flic->nzx;
			v = rav_array[ir][1]/
				rav_array[ir][0];
			wvert = rav_array[ir][2]/
				rav_array[ir][0];
 			shaft = shaft_max * sqrt(v*v + (wvert *flic->w_exag) *
		    		(wvert *flic->w_exag));
		  	xplot_arrow(XtDisplay(flic->widget), 
		  		XtWindow(flic->widget), gc, x, y, shaft, v, 
		  		wvert * flic->w_exag);
		  	/* also plot to (global) pixmap so can scroll */
		  	xplot_arrow(XtDisplay(flic->widget), 
		  		pixmap, gc, x, y, shaft, v, 
		  		wvert * flic->w_exag);
	   	}
	   }

	  } /* Finished  one level */
/* Set back to Black, as default */
	set_color(flic->widget, "Black");
/* dealy-bobber to show scaling of arrows: */
	   x = flic->pic_left + flic->pic_wid + 10.0;
	   y = flic->height - (  flic->pic_bottom + flic->pic_ht + 10.0);
	   shaft = shaft_max * 5.0 * flic->w_exag;
	   xplot_arrow(XtDisplay(flic->widget), 
	  		XtWindow(flic->widget), gc, x, y, shaft, 0.0, 
	  		5.0 * flic->w_exag );
	   xplot_arrow(XtDisplay(flic->widget), 
	  		pixmap, gc, x, y, shaft, 0.0, 
	  		5.0 * flic->w_exag );
	   shaft = shaft_max * 5.0;
	   xplot_arrow(XtDisplay(flic->widget), 
	  		XtWindow(flic->widget), gc, x, y, shaft, 5.0, 0.0);
	   xplot_arrow(XtDisplay(flic->widget), 
	  		pixmap, gc, x, y, shaft, 5.0, 0.0);
	   len = sprintf(string,"5 m/s\n");
	   XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x+5, y-5, string, len);
	   XDrawString(XtDisplay(flic->widget), XtWindow(flic->widget),
			gc, x+5, y-5, string, len);

	}	  
/* Added azimuth-height wind overlay */
/* Az-Z mean slices ... a little more work */
	else if ( flic->orientation == AZ_SLICE) { 
	  xc = flic->head->xz;
	  yc = flic->head->yz;
	  flic->nzx = flic->pic_wid / RMAX ; /* RMAX is dim of radial avg array */
/* Typically the lowest layer in a Doppler analysis is above the ground.
 * If so set nzy smaller, to hold that blank strip at the bottom...
 */
	  if( flic->head->zz > 0.0 ) {
  		flic->nzy = flic->pic_ht / (1 + flic->kdim ) ;
		kcorn = 1;
	  }
	  else {
	  	flic->nzy = flic->pic_ht / flic->kdim;
	  	kcorn = 0;
	  }
	  speed_max = 10.0;
	  shaft_max = flic->arrow_shaft_length / speed_max;
	  d_rav = (flic->az_max - flic->az_min) / (RMAX-1);
	  flic->w_exag = 2.0 * 
	  	( d_rav / flic->nzx) / (flic->head->szt / flic->nzy);
	  radius = flic->level;
 	  for(k = 0; k < flic->kdim; k++ ) {
	    y = flic->height - (  flic->pic_bottom + (k+kcorn) * flic->nzy);
	    for (i = 0; i < RMAX; i++) {
	    		rav_array[i][0] = 0.0;
	    		rav_array[i][1] = 0.0;
	    		rav_array[i][2] = 0.0;
	    		rav_array[i][3] = 0.0;
	    }
            for( ia = 0; ia < RMAX; ia++) {
            	az = flic->az_min + ia * d_rav;
            	xp = xc + radius * cos(az);
            	yp = yc + radius * sin(az);
		if( flic->shift_center ) {
			xp = xp + flic->cen_shift[0];
			yp = yp + flic->cen_shift[1];
		}
		i = xp / flic->head->sxt;
		j = yp / flic->head->syt;
	    	if (i>= 0 && i<flic->idim && j>= 0 && j<flic->jdim) {
			wdir = flic->data[k][j][i][0] * 0.1;
			wspd = flic->data[k][j][i][1] * 0.1;
			wvert = flic->data[k][j][i][2] * 0.01;
			if ( wvert > bad_w  ){
			 if(wdws_to_vtvr(wdir, wspd, xp, yp, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1) {
			   rav_array[RMAX-1-ia][0] = 
			   	rav_array[RMAX-1-ia][0] + 1.0;
			   rav_array[RMAX-1-ia][1] = 
			   	rav_array[RMAX-1-ia][1] + vt;
			   rav_array[RMAX-1-ia][2] = 
			   	rav_array[RMAX-1-ia][2] + wvert;
			 }
			}
		} 
	     }

	   /* now plot the mean winds at this level */
	   for ( ir = 0; ir < RMAX; ir++ ) {
	   	if( rav_array[ir][0] > 0.0 ) {
	   		x = flic->pic_left + ir * flic->nzx;
/* - sign so tangential winds > 0 go right to left, 
 * as would be seen from inside. */
			v = -rav_array[ir][1]/
				rav_array[ir][0];
			wvert = rav_array[ir][2]/
				rav_array[ir][0];
 			shaft = shaft_max * sqrt(v*v + (wvert *flic->w_exag) *
		    		(wvert *flic->w_exag));
		  	xplot_arrow(XtDisplay(flic->widget), 
		  		XtWindow(flic->widget), gc, x, y, shaft, v, 
		  		wvert * flic->w_exag);
		  	xplot_arrow(XtDisplay(flic->widget), 
		  		pixmap, gc, x, y, shaft, v, 
		  		wvert * flic->w_exag);
	   	}
	   }

	  } /* end one level */
/* Set back to Black, as default */
	set_color(flic->widget, "Black");
/* dealy-bobber to show scaling of arrows: */
	   x = flic->pic_left + flic->pic_wid + 10.0;
	   y = flic->height - (  flic->pic_bottom + flic->pic_ht + 10.0);
	   shaft = shaft_max * 5.0 * flic->w_exag;
	   xplot_arrow(XtDisplay(flic->widget), 
	  		XtWindow(flic->widget), gc, x, y, shaft, 0.0, 
	  		5.0 * flic->w_exag );
	   xplot_arrow(XtDisplay(flic->widget), 
	  		pixmap, gc, x, y, shaft, 0.0, 
	  		5.0 * flic->w_exag );
	   shaft = shaft_max * 5.0;
	   xplot_arrow(XtDisplay(flic->widget), 
	  		XtWindow(flic->widget), gc, x, y, shaft, 5.0, 0.0);
	   xplot_arrow(XtDisplay(flic->widget), 
	  		pixmap, gc, x, y, shaft, 5.0, 0.0);
	   len = sprintf(string,"5 m/s\n");
	   XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x+5, y-5, string, len);
	   XDrawString(XtDisplay(flic->widget), XtWindow(flic->widget),
			gc, x+5, y-5, string, len);

	}
/* Added arbitrary slice wind overlay */
	else if ( flic->orientation == ARB_SLICE) { 
	  xc = flic->head->xz;
	  yc = flic->head->yz;
	  flic->nzx = flic->pic_wid / RMAX ;
/* Typically the lowest layer in a Doppler analysis is above the ground.
 * If so set nzy smaller, to hold that blank strip at the bottom...
 */
	  if( flic->head->zz > 0.0 ) {
  		flic->nzy = flic->pic_ht / (1 + flic->kdim ) ;
		kcorn = 1;
	  }
	  else {
	  	flic->nzy = flic->pic_ht / flic->kdim;
	  	kcorn = 0;
	  }
	  speed_max = 10.0;
	  shaft_max = flic->arrow_shaft_length / speed_max;
	  radius = sqrt( (flic->x_end - flic->x_beg)*(flic->x_end - flic->x_beg) +
	  	(flic->y_end - flic->y_beg)*(flic->y_end - flic->y_beg) );
	  d_s = radius / (RMAX-1);
	  az = atan2((flic->y_end - flic->y_beg),
	  	(flic->x_end - flic->x_beg));
	  flic->w_exag = (flic->head->sxt / flic->nzx) / 
	  	(flic->head->szt / flic->nzy);
/* comment out	  flic->w_exag = 2.0 * 
	  	( d_s / flic->nzx) / (flic->head->szt / flic->nzy);
 */
 	  for(k = 0; k < flic->kdim; k++ ) {
	    y = flic->height - (  flic->pic_bottom + (k+kcorn) * flic->nzy);
	    for (i = 0; i < RMAX; i++) {
	    		rav_array[i][0] = 0.0;
	    		rav_array[i][1] = 0.0;
	    		rav_array[i][2] = 0.0;
	    		rav_array[i][3] = 0.0;
	    }
            for( ia = 0; ia < RMAX; ia++) {
            	xp = flic->x_beg + ia * d_s * cos(az);
            	yp = flic->y_beg + ia * d_s * sin(az);
		if( flic->shift_center ) {
			xp = xp + flic->cen_shift[0];
			yp = yp + flic->cen_shift[1];
		}
		i = floor(xp);
		if( fabs(i-xp) > 0.5) i = ceil(xp);
		j = floor(yp);
		if( fabs( j - yp) > 0.5) j = ceil(yp);

	    	if (i>= 0 && i<flic->idim && j>= 0 && j<flic->jdim) {
			wdir = flic->data[k][j][i][0] * 0.1;
			wspd = flic->data[k][j][i][1] * 0.1;
			wvert = flic->data[k][j][i][2] * 0.01;
			if ( wvert > bad_w  ){
			 if(wdws_to_vtvr(wdir, wspd, xp, yp, xc, yc, 
				flic->shift_center,
				flic->cen_shift[0], flic->cen_shift[1], 
				flic->remove_motion,
				flic->storm_uv[0], flic->storm_uv[1], 
				&u, &v, &vr, &vt, bad_val) != -1) {
			   rav_array[ia][0] = 1.0;
/* projection of horizontal wind on slice */
			   rav_array[ia][1] = u*cos(az) + v*sin(az);
			   rav_array[ia][2] = wvert;
			 }
			}
		} 
	     }

	   /* now plot the mean winds at this level */
	   for ( ir = 0; ir < RMAX; ir++ ) {
	   	if( rav_array[ir][0] > 0.0 ) {
	   		x = flic->pic_left + ir * flic->nzx;
			v = rav_array[ir][1];
			wvert = rav_array[ir][2];				rav_array[ir][0];
 			shaft = shaft_max * sqrt(v*v + (wvert *flic->w_exag) *
		    		(wvert *flic->w_exag));
		  	xplot_arrow(XtDisplay(flic->widget), 
		  		XtWindow(flic->widget), gc, x, y, shaft, v, 
		  		wvert * flic->w_exag);
		  	xplot_arrow(XtDisplay(flic->widget), 
		  		pixmap, gc, x, y, shaft, v, 
		  		wvert * flic->w_exag);
	   	}
	   }

	  } /* Finished  one level */
/* Set back to Black, as default */
	set_color(flic->widget, "Black");
/* dealy-bobber to show scaling of arrows: */
	   x = flic->pic_left + flic->pic_wid + 10.0;
	   y = flic->height - (  flic->pic_bottom + flic->pic_ht + 10.0);
	   shaft = shaft_max * 5.0 * flic->w_exag;
	   xplot_arrow(XtDisplay(flic->widget), 
	  		XtWindow(flic->widget), gc, x, y, shaft, 0.0, 
	  		5.0 * flic->w_exag );
	   xplot_arrow(XtDisplay(flic->widget), 
	  		pixmap, gc, x, y, shaft, 0.0, 
	  		5.0 * flic->w_exag );
	   shaft = shaft_max * 5.0;
	   xplot_arrow(XtDisplay(flic->widget), 
	  		XtWindow(flic->widget), gc, x, y, shaft, 5.0, 0.0);
	   xplot_arrow(XtDisplay(flic->widget), 
	  		pixmap, gc, x, y, shaft, 5.0, 0.0);
	   len = sprintf(string,"5 m/s\n");
	   XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x+5, y-5, string, len);
	   XDrawString(XtDisplay(flic->widget), XtWindow(flic->widget),
			gc, x+5, y-5, string, len);

	}
/* Set line widths back to 0, for faster drawing */
/*        printf("WindOverlay before setlineattributes \n"); */
	XSetLineAttributes(XtDisplay(flic->widget), gc, 0, LineSolid,
		CapButt, JoinBevel);
/* Add caption and grid */
/*        printf("After setlineattributes \n"); */
	Grid(flic->widget, flic, call_data);
/*        printf("After Grid \n"); */
	Caption(flic->widget, flic, call_data);	

/* render pixmap into window */
/*        printf("Before XCopyArea \n"); */
	XCopyArea(XtDisplay(flic->widget), pixmap, XtWindow(flic->widget),
		gc, 0, 0, flic->width, flic->height, 0, 0);
/*        printf("After XCopyArea \n"); */
}

/* Following function is lifted from O'Reilly Vol 6, Chapter 15, 
 * example 15-2, with
 * modifications for my use... */
 
void file_cb( w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;
{
	static Widget dialog;
	/* cast required in ANSI C */
	struct chitra *flic = client_data;

	void get_file();
	
	if( !dialog ) {
		Widget menu = XtParent(w);
		dialog = XmCreateFileSelectionDialog(menu, "file_sb",
			NULL, 0);
		XtAddCallback(dialog, XmNokCallback, get_file, client_data);
		XtAddCallback(dialog, XmNcancelCallback, 
			(XtCallbackProc) XtUnmanageChild, 
			NULL);
	}
	XtManageChild(dialog);
	
	XtPopup(XtParent(dialog), XtGrabNone);
	/* If dialog is already popped up, XtPopup does nothing.
	 * call XMapRaised() anyway to make sure it is visible
	 */
/* following commented out so I can put file requester window behind
 * the other windows... */
 
/* Commented out:
	XMapRaised(XtDisplay(dialog), XtWindow(XtParent(dialog)));
 */
}

void get_file( w, client_data, cbs)
Widget w;
XtPointer client_data;
XmFileSelectionBoxCallbackStruct *cbs;
{
	/* cast required in ANSI C */
	struct chitra *flic = client_data;

	char *filename;
	FILE *infile;
	Widget file_w;
	void read_file();
	static Widget dialog;

	if(XtIsSubclass(w, xmTextFieldWidgetClass)) {
		filename = XmTextFieldGetString(w);
		file_w = w; /* this is the file widget */
	}
	else {
		/* file was selected from FileSelectionDialog */
		XmStringGetLtoR(cbs->value, XmSTRING_DEFAULT_CHARSET,
			&filename);
	}
	
	if( !filename || !*filename) {  /* nothing typed? */
		if (filename)
			XtFree(filename);
		return;
	}
	
/* open file */	
	printf(" Will try to open %s.\n",filename);
	infile = fopen( filename, "r");
	if ( infile == NULL ) {
		printf("Can't open file %s\n", filename);
		return;
	}

/* Note: When I call this in the main (radar_slicer.c) I call as
 * read_file(infile, &flic) because there flic is declared a structrue
 * and read_file expects a pointer to that structure. However in this function
 * the variable flic IS already a pointer to the structure, since it has been
 * passed thought a couple of Xt callbacks, so I don't need the &flic.
 */
        printf("calling read_file \n");
	read_file( infile, flic );
        printf("returned from read_file \n");
	/*	fclose( infile); */
        printf("returned from infile \n");
}

void read_file ( FILE *infile, struct chitra *flic)
{
	static struct hdr radar_hdr;
	void show_header(), get_data(), get_rmin();
	
/* Note that we pass the address of the structure, not the structure*/
	show_header( infile, &radar_hdr);

/* we pass address of structure so get_data can read keywd and decide
 * how to read in data. I suppose could just pass in ptr to keywd.. */
	get_data( infile, &radar_hdr, &flic->data);
	fclose( infile);
/* Now load up some of structure flic that holds info for drawing */
	flic->idim = radar_hdr.idim;
	flic->jdim = radar_hdr.jdim;
	flic->kdim = radar_hdr.kdim;
	flic->head = &radar_hdr;
        printf("calling get_rmin \n");
	get_rmin(radar_hdr.idim, radar_hdr.jdim, -radar_hdr.xz, -radar_hdr.yz, 
		radar_hdr.sxt, radar_hdr.syt, &flic->rad_min, &flic->rad_max,
		&flic->az_min, &flic->az_max);
        printf("returned from get_rmin \n");
	flic->x_beg = 0.0;
	flic->y_beg = 0.0;
	flic->x_end = radar_hdr.idim * radar_hdr.sxt;
	flic->y_end = radar_hdr.jdim * radar_hdr.syt;
}


void show_header( FILE *infile, struct hdr *rdinfo)
{
	int num_read, rec_len;
	size_t hdr_siz;

	num_read =  fread( &rec_len, sizeof(int), 1, infile);
/*	printf(" Header record length is %d .\n", rec_len);
 */
	hdr_siz = rec_len;
/* Note that because rdinfo is declared as a pointer to structure hdr
 * we don't want to pass it's address...because it already IS the address. */
 
	num_read =  fread( rdinfo, hdr_siz, 1, infile);
	if(num_read == NULL) {
		printf("error reading file.");
		exit(EXIT_FAILURE);
	}
	printf(" File type: %.4s\n", rdinfo->keywd );
	if( strncmp( rdinfo->keywd, "wind", 4) == 0 ) {
		printf(" Changing to type WIND ");
		strncpy(rdinfo->keywd, "WIND", 4);
	}
	if( strncmp( rdinfo->keywd, "cmp ", 4) == 0 ) {
		printf(" Changing to type CMP ");
		strncpy(rdinfo->keywd, "CMP ", 4);
	}
	if( strncmp( rdinfo->keywd, "swp ", 4) == 0 ) {
		printf(" Changing type swp to type CMP ");
		strncpy(rdinfo->keywd, "CMP ", 4);
	}
	printf(" Flight: %.8s\n", rdinfo->flight );
	printf(" Storm: %.12s\n", rdinfo->storm );
	printf(" Radar ID: %.4s\n",rdinfo->radarid);
	printf(" Track file: %.32s\n",rdinfo->trkfil);
	printf(" Created on: %.32s\n",rdinfo->create_time);
	printf(" Ram file: %.28s\n",rdinfo->namram);
	printf(" Array dimensions (i,j,k) : %d %d %d \n",
		rdinfo->idim, rdinfo->jdim, rdinfo->kdim);
	printf(" Grid box size: %.2f km by %.2f km by %.2f km. \n",
		rdinfo->sxt, rdinfo->syt, rdinfo->szt);
	if ( rdinfo->nmosm == 0 ){
		printf(" Coordinate system is earth relative.\n");
		printf(" Lower left corner: %.2f km, %.2f km, %.2f km. \n",
			rdinfo->xz, rdinfo->yz, rdinfo->zz);
		}
	else
		{
		printf(" Coordinate system is storm relative. \n");
		printf(" Storm location in grid: %.2f km, %.2f km, %.2f km. \n",
			rdinfo->xz, rdinfo->yz, rdinfo->zz);
	}
		
	printf(" Coordinate system rotated %.2f degrees.\n",
		rdinfo->rot);
/* read last four bytes of header record */
	num_read =  fread( &rec_len, sizeof(int), 1, infile);
}
 
/* Function to read in data part of HRD Doppler file */
/* only reads in wind data at present */

void get_data(FILE *infile, struct hdr *rdinfo, 
	int short data[KMAX][JMAX][IMAX][5] )

{
	char data_type[4];
	int num_read, rec_len;
	size_t num_elem;
	int short data_buf[2048];	
	int i, j, k, nvar, iv, iptr;
	unsigned int short ipacked, val_dbz;
	float dbz;
	
	strncpy(data_type, rdinfo->keywd, 4);
	printf(" data format is %.4s . \n",data_type);
	if (strncmp( data_type, "DOP3", 4) == 0) 
		printf(" can't read %s type of data yet.\n",data_type);	   
	else if (strncmp(data_type, "CMP ",4) == 0 ){
		iv = 3; /* save dBZ in 4th variable of data array */
/* now read data_buffers, load into data array  */
		k = 0; /* only one level in cmp files */
/* unusual loop for j because CMP files stored in opposite order from other
 * files. */
		for (j = rdinfo->jdim-1; j > -1; j--)
		{
/* printf(" going for record k, j %d %d .\n", k, j);  */
/* 3 freads per record because of how hpux f77 writes unformatted files */
/* read size in first four bytes */
		    num_read =  fread( &rec_len, sizeof(int), 1, infile);
		    if ( num_read != NULL ){
/* read record */
			num_elem = rec_len / sizeof(data_buf[0]);
			num_read =  fread( &data_buf[0], sizeof(data_buf[0]), 
				num_elem, infile);
/* read last four bytes of this record ( which also holds # bytes) */
			num_read =  fread( &rec_len, sizeof(int), 1, infile);
/* i know this unpacking code is ugly.. but i needed it NOW */
			for(i = 0; i < rdinfo->idim-1; i+=2 )
			{
				iptr = i / 2 ;
				ipacked = data_buf[iptr];
				val_dbz = ipacked & 255;
				if( val_dbz == 0 ) 
					dbz = -999.9;
				else
					dbz = (val_dbz -64) * 0.5;
				data[k][j][i+1][iv] = dbz * 10.0;
				val_dbz = ipacked >> 8;
				if( val_dbz == 0  )
					dbz = -999.9;
				else
					dbz = (val_dbz - 64) * 0.5;
				data[k][j][i][iv] = dbz * 10.0;
			}

		     }
		     else if (feof(infile) != 0 )
			printf( " end of file \n");
		     else
			perror(NULL);
		} /* j for */
/* Put bad_val in other elements, so can't draw wind barbs by accident: */
		for (j = 0; j < rdinfo->jdim; j++) {
			for (i = 0; i < rdinfo->idim; i++) {
				data[k][j][i][0] = -9999;
				data[k][j][i][1] = -9999;
				data[k][j][i][2] = -9999;
				data[k][j][i][4] = -9999;
			}
		}
	} /* End CMP reading block */
	else if (strncmp(data_type, "TA3D",4) == 0  ) {
	iv = 3; /* save dBZ in 4th variable of data array */
/* now read data_buffers, load into data array  */
	for (k = 0; k < rdinfo->kdim; k++)
	{
		for (j = 0; j < rdinfo->jdim; j++)
		{
/* printf(" going for record k, j %d %d .\n", k, j);  */
/* 3 freads per record because of how hpux f77 writes unformatted files */
/* read size in first four bytes */
		    num_read =  fread( &rec_len, sizeof(int), 1, infile);
		    if ( num_read != NULL ){
/* read record */
			num_elem = rec_len / sizeof(data_buf[0]);
			num_read =  fread( &data_buf[0], sizeof(data_buf[0]), 
				num_elem, infile);
/* read last four bytes of this record ( which also holds # bytes) */
			num_read =  fread( &rec_len, sizeof(int), 1, infile);
/* i know this unpacking code is ugly.. but i needed it NOW */
			for(i = 0; i < rdinfo->idim-1; i+=2 )
			{
				iptr = i / 2 ;
				ipacked = data_buf[iptr];
				val_dbz = ipacked & 255;
				if( val_dbz == 0 ) 
					dbz = -999.9;
				else
					dbz = (val_dbz -64) * 0.5;
				data[k][j][i+1][iv] = dbz * 10.0;
				val_dbz = ipacked >> 8;
				if( val_dbz == 0  )
					dbz = -999.9;
				else
					dbz = (val_dbz - 64) * 0.5;
				data[k][j][i][iv] = dbz * 10.0;
			}

		     }
		     else if (feof(infile) != 0 )
			printf( " end of file \n");
		     else
			perror(NULL);
		} /* j for */
	} /* k for */
/* Put bad_val in other elements, so can't draw wind barbs by accident: */
	for (k = 0; k < rdinfo->kdim; k++) {
		for (j = 0; j < rdinfo->jdim; j++) {
			for (i = 0; i < rdinfo->kdim; i++) {
				data[k][j][i][0] = -9999;
				data[k][j][i][1] = -9999;
				data[k][j][i][2] = -9999;
				data[k][j][i][4] = -9999;
			}
		}
	}
	} /* End TA3D reading block */
	else if( strncmp(data_type, "WIND", 4) == 0 )
		{
	nvar = 5; /* 5 variables per bin in wind arrays */
/* now read data_buffers, load into data array  */
	for (k = 0; k < rdinfo->kdim; k++)
	{
		for (j = 0; j < rdinfo->jdim; j++)
		{
/* printf(" going for record k, j %d %d .\n", k, j);  */
/* 3 freads per record because of how hpux f77 writes unformatted files */
/* read size in first four bytes */
		    num_read =  fread( &rec_len, sizeof(int), 1, infile);
		    if ( num_read != NULL ){
/* read record */
			num_elem = rec_len / sizeof(data_buf[0]);
			num_read =  fread( &data_buf[0], sizeof(data_buf[0]), 
				num_elem, infile);
/* read last four bytes of this record ( which also holds # bytes) */
			num_read =  fread( &rec_len, sizeof(int), 1, infile);
			for(i = 0; i < rdinfo->idim; i++)
			{
				iptr = i * nvar;
				for( iv =0; iv < nvar; ++iv)
				  data[k][j][i][iv] = 
				  	data_buf[iptr +iv];
			}

		     }
		     else if (feof(infile) != 0 )
			printf( " end of file \n");
		     else
			perror(NULL);
		} /* j for */
	} /* k for */
	} /* End WIND reading block */
	
	else
		printf(" what type of data are in %.4s files?\n",rdinfo->keywd);
	return;
}

void Caption(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;

{
	struct chitra *flic = client_data;
	int i, j, k, iv, iwd, iht, nskip, ir;
	Position x, y;
	double shaft;
	float wdir, wspd, u, v,	xp, yp, xc, yc, ysq;
	
	char string[48];
	int len, ihs, ims, iss, ihe, ime, ise;

/*        printf("Entered Caption \n"); */
	ihs = flic->head->stime1 / 3600.0;
	ims = (flic->head->stime1 - ihs * 3600) / 60.0;
	iss = flic->head->stime1 - ihs * 3600 - ims * 60;
	ihe = flic->head->etime2 / 3600.0;
	ime = (flic->head->etime2 - ihe * 3600) / 60.0;
	ise = flic->head->etime2 - ihe * 3600 - ime * 60;
/*        printf("Caption Time \n"); */
	len = sprintf(string,"%.8s %.12s %.2d:%.2d:%.2d - %.2d:%.2d:%.2d UTC\n",
		 flic->head->flight,
		flic->head->storm, ihs,ims,iss,ihe,ime,ise); 
/*        printf("Caption before x and y \n"); */
/*        printf("A \n"); */
	x = flic->pic_left;
/*        printf("B \n"); */
/*        printf("Caption after x and before y \n"); */
	y = flic->height - (flic->pic_bottom + flic->pic_ht + 
		flic->font_str->ascent + 2 * flic->font_str->descent);
/*        printf("Caption after x and y \n"); */
	XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x, y, string, len);

	if( flic->var_type == REFLECTIVITY)
		len = sprintf(string,"Reflectivity \n");
	else if( flic->var_type == WINDSPEED)
		len = sprintf(string,"Total Wind Speed \n");
	else if( flic->var_type == VERTWINDSPEED)
		len = sprintf(string,"Vertical Wind Speed\n");
	else if( flic->var_type == RADWIND)
		len = sprintf(string,"Radial Wind Speed \n");
	else if( flic->var_type == TANWIND)
		len = sprintf(string,"Tangential Wind Speed \n");
	else if( flic->var_type == U_COMPONENT)
		len = sprintf(string,"U component \n");
	else if( flic->var_type == V_COMPONENT)
		len = sprintf(string,"V component \n");

	y = flic->height - (flic->pic_bottom + flic->pic_ht
		+ flic->font_str->descent);
/*         printf("Caption 3\n"); */
	XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x, y, string, len);

	if ( flic->orientation == XY_SLICE)
	      len = sprintf(string, "XY Slice at Z = %.1f km\n",
	      	flic->head->zz + flic->level * flic->head->szt);
        else if( flic->orientation == XZ_SLICE)
	      len = sprintf(string, "XZ Slice at Y = %.1f km\n",
	      	(flic->level + 1) * flic->head->syt);
        else if( flic->orientation == YZ_SLICE)
	      len = sprintf(string, "YZ Slice at X = %.1f km\n",
	      	(flic->level + 1) * flic->head->sxt);
        else if( flic->orientation == RZ_SLICE)
	      len = sprintf(string, "RZ Slice\n");
        else if( flic->orientation == AZ_SLICE)
	      len = sprintf(string, "Az-Z Slice at %.1f km radius\n",
	      flic->level*1.0);
        else if( flic->orientation == ARB_SLICE)
	      len = sprintf(string, "Slice from (%.1f, %.1f) to (%.1f, %.1f)\n",
	      flic->x_beg, flic->y_beg, flic->x_end, flic->y_end);
	      
	x = flic->pic_left + flic->pic_wid -
		XTextWidth(flic->font_str, string, len);
/*        printf("Caption 4\n"); */
	XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x, y, string, len);

			
	x = flic->pic_left + 0.5 * flic->pic_wid;
	if( flic->shift_center ) {
		len = sprintf(string,"Center shifted %.1f, %.1f km\n",
			flic->cen_shift[0],flic->cen_shift[1]);
		y = flic->height - (flic->pic_bottom + flic->pic_ht + 
			flic->font_str->ascent + 2 * flic->font_str->descent);
		XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x, y, string, len);
	}

	if( flic->remove_motion ) {
		len = sprintf(string,"Removed %.1f, %.1f m/s\n",
			flic->storm_uv[0],flic->storm_uv[1]);
		y = flic->height - (flic->pic_bottom + flic->pic_ht + 
		  2 *(flic->font_str->ascent + 2 * flic->font_str->descent) );
		XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x, y, string, len);
	}
/*        printf("Leaving Caption \n"); */
}

void ColorScale(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;

{
	struct chitra *flic = client_data;
	int i, x, y, v1, v2;
	float zscale, zmin, zmax;
	char string[12];
	int len, box_wid, box_ht;
	void set_color();
/* pixmap and colors are global */
	sprintf(string,"m/s\n");
	zmin = flic->var_scale[flic->var_type][0];
	zmax = flic->var_scale[flic->var_type][1];
	
	if( flic->var_type == REFLECTIVITY) {
		sprintf(string,"dBZ\n");
	}

	zscale = (XtNumber(colors) - 1) / (zmax - zmin);
	box_wid = 13; /* These should depend on the size available */
	box_ht = 22;

/* Commented out because maybe not working on Ed Brackens SUN
	XSetForeground(XtDisplay(flic->widget), gc, col_pix[0]);
 */
	set_color(flic->widget, "Black");
	x = flic->pic_left + flic->pic_wid + 10; /* REPLACE MAGIC # !! */
	y = flic->height - (flic->pic_bottom + XtNumber(colors) * box_ht);
	XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x, y, string, 3);
	for (i = 0; i < XtNumber(colors) ; i++) {
		y = flic->height - (flic->pic_bottom + (i+1) * box_ht);
		XSetForeground(XtDisplay(flic->widget), gc, col_pix[i]);
		XFillRectangle( XtDisplay(flic->widget), pixmap,
			gc, x, y, box_wid, box_ht);
	}		

/* Commented out because maybe not working on Ed Brackens SUN
	XSetForeground(XtDisplay(flic->widget), gc, col_pix[0]);
 */
	set_color(flic->widget, "Black");
	for (i = 0; i < XtNumber(colors) ; i++) {
		y = flic->height - (flic->pic_bottom + (i+1) * box_ht);
		v1 = ( i / zscale) + zmin;
		v2 = ( (i+1) / zscale) + zmin -1;
		y = y + box_ht;
		if( (len = sprintf(string,"%d - %d\n", v1, v2) ) > 0 )
		   XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x+box_wid+2, y, string, len);
	}		

}

void Grid(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;

{
	struct chitra *flic = client_data;
	Position xb, yb, xe, ye, x, y;
	int xc, yc, i, j, ileft, iright, jbot, jtop, iskip, jskip;
	int del_str, iskip_ch, jskip_ch;
	int ch_wid;
	int width, height;
	float sx, sy; /* km to pixel scale factors */
	float radius, d_rav;
	float az, az_met;	
	char string[8];
	int len;
	
/* get width of grid labels */
	sprintf(string, "123\n");
	del_str = XTextWidth(flic->font_str, string, strlen(string));
/* Note scaling IS 
 * km to pixels; that is i and j refer to km, x,y pixels. */
	if ( flic->orientation == XY_SLICE) { /* XY slices */
	  flic->nzx = flic->pic_wid / flic->idim ;
	  flic->nzy = flic->pic_ht / flic->jdim ;
	  if( flic->nzx < flic->nzy) flic->nzy = flic->nzx; /* square up */
	  flic->nzx = flic->nzy;
	  width = flic->nzx * flic->idim;
	  height = flic->nzy * flic->jdim;
	  sx = flic->nzx / flic->head->sxt;
	  sy = sx;
	  iskip = 5;
	  jskip = 5;
	  if( flic->head->sxt * flic->idim <= 90.0 ) {
		iskip_ch = 15;
		jskip_ch = 15;
	  }
	  else {
		iskip_ch = 50;
		jskip_ch = 50;
	  }
	  iright = (flic->head->sxt * flic->idim) - flic->head->xz;
	  ileft = -flic->head->xz;
	  jtop = (flic->head->syt * flic->jdim) - flic->head->yz;
	  jbot = -flic->head->yz;
	  xc = -ileft * sx;
	  yc = -jbot * sy;
	}
	else if ( flic->orientation == XZ_SLICE) { /* XZ slices */
	  flic->nzx = flic->pic_wid / flic->idim ;
	  flic->nzy = flic->pic_ht / flic->kdim ;
	  width = flic->nzx * flic->idim;
	  height = flic->nzy * flic->kdim;
	  sx = flic->nzx / flic->head->sxt;
	  sy = flic->nzy / flic->head->szt;
	  iskip = 5;
	  jskip = 1;
	  iskip_ch = 15;
	  jskip_ch = 1;
	  iright = (flic->head->sxt * flic->idim) - flic->head->xz;
	  ileft = -flic->head->xz;
	  jbot = 1;
	  jtop = flic->kdim * flic->head->szt;
	  xc = -ileft * sx;
	  yc = 0; 
	}	
	else if ( flic->orientation == YZ_SLICE) { /* YZ slices */
	  flic->nzx = flic->pic_wid / flic->jdim ;
	  flic->nzy = flic->pic_ht / flic->kdim ;
	  width = flic->nzx * flic->jdim;
	  height = flic->nzy * flic->kdim;
	  sx = flic->nzx / flic->head->syt;
	  sy = flic->nzy / flic->head->szt;
	  iskip = 5;
	  jskip = 1;
	  iskip_ch = 15;
	  jskip_ch = 1;
	  iright = (flic->head->syt * flic->jdim) - flic->head->yz;
	  ileft = -flic->head->yz;
	  jbot = 1;
	  jtop = flic->kdim * flic->head->szt;
	  xc = -ileft * sx;
	  yc = 0;
	}
	else if ( flic->orientation == RZ_SLICE) { /* RZ slices */
	  flic->nzx = flic->pic_wid / RMAX;
	  flic->nzy = flic->pic_ht / flic->kdim ;
	  d_rav = (flic->rad_max - flic->rad_min) / (RMAX-1);
	  width = flic->nzx * RMAX;
	  height = flic->nzy * flic->kdim;
	  sx = flic->nzx / d_rav;
	  sy = flic->nzy / flic->head->szt;
	  iskip = 5;
	  jskip =1;
	  iskip_ch = 10; /* label every 10 km in radius */
	  jskip_ch = 1;  /* label every dz (usually 1 km) in height */
	  ileft = flic->rad_min;
	  iright = flic->rad_max;
	  xc = -ileft * sx; /* Because rmin not necc 0.0 */
	  yc = 0;
	  jbot = 1;
	  jtop = flic->kdim * flic->head->szt;
	}
	else if ( flic->orientation == AZ_SLICE) { /* AZ - height slices */
	  flic->nzx = flic->pic_wid / RMAX;
	  d_rav = (flic->az_max - flic->az_min) / (RMAX-1);
	  width = flic->nzx * RMAX;
	  flic->nzy = flic->pic_ht / flic->kdim ;
	  height = flic->nzy * flic->kdim;
	  sy = flic->nzy / flic->head->szt;
	  jskip =1;
	  jskip_ch = 1;  /* label every dz (usually 1 km) in height */
	  yc = 0;
	  jbot = 1;
	  jtop = flic->kdim * flic->head->szt;
	}
	else if ( flic->orientation == ARB_SLICE) { /* Arbitrary slices */
	  flic->nzx = flic->pic_wid / RMAX;
	  width = flic->nzx * RMAX;
	  flic->nzy = flic->pic_ht / flic->kdim ;
	  height = flic->nzy * flic->kdim;
	  sy = flic->nzy / flic->head->szt;
	  jskip =1;
	  jskip_ch = 1;  /* label every dz (usually 1 km) in height */
	  xc = 0; /* Because rmin not necc 0.0 */
	  yc = 0;
	  jbot = 1;
	  jtop = flic->kdim * flic->head->szt;
	}
/* First draw bounding box: */
	xb = flic->pic_left;
	yb = flic->height - (flic->pic_bottom + height);
	xe = xb + width;
	ye = flic->height - flic->pic_bottom;
	
	XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, yb, xe, yb);
	XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, ye, xe, ye);
	XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, yb, xb, ye);
	XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xe, yb, xe, ye);


/* Now do the grids  and labels */
/* X, radial, azimuthal or arbitrary sliceaxis */
	yb = flic->height - flic->pic_bottom;
	ye = flic->height -(flic->pic_bottom + height);

	if ( flic->orientation == AZ_SLICE) { /* AZ - height slices */
/* Have to number right to left */
	  sx = flic->nzx;
	  iskip = 5;
	  iskip_ch = 15; /* label every 10 deg in az */
	   for (i = 0; i < RMAX; i+=iskip) {
	    	xb = flic->pic_left + flic->pic_wid  - i * sx;
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, yb, xb, yb-6);
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, ye, xb, ye+6);
	   }
	   for (i = 0; i < RMAX; i+=iskip_ch) {
	    	xb = flic->pic_left + flic->pic_wid - i * sx;
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, yb, xb, yb-12);
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, ye, xb, ye+12);
		y = yb +
		   (flic->font_str->ascent + flic->font_str->descent);
		az = flic->az_min + i * d_rav;
		az_met = 90.0 - (az * 180.0 / acos(-1.0) );
		if( az_met < 0.0 ) az_met = az_met + 360.0;
		len = sprintf(string, "%d", (int) az_met);
		ch_wid = XTextWidth(flic->font_str, string, len);
	    	xb = xb - ch_wid/2 ;
		XDrawString(XtDisplay(flic->widget), pixmap,
			gc, xb, y, string, len);
	   }
	}
	else if ( flic->orientation == ARB_SLICE) { 
	  sx = flic->nzx;
	  iskip = 5;
	  iskip_ch = 15; /* label every 10 deg in az */
	   for (i = 0; i < RMAX; i+=iskip) {
	    	xb = flic->pic_left  + xc + i * sx;
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, yb, xb, yb-6);
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, ye, xb, ye+6);
	   }
	   for (i = 0; i < RMAX; i+=iskip_ch) {
	    	xb = flic->pic_left + xc + i * sx;
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, yb, xb, yb-12);
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, ye, xb, ye+12);
		y = yb +
		   (flic->font_str->ascent + flic->font_str->descent);
		len = sprintf(string, "%d", i);
		ch_wid = XTextWidth(flic->font_str, string, len);
	    	xb = xb - ch_wid/2 ;
		XDrawString(XtDisplay(flic->widget), pixmap,
			gc, xb, y, string, len);
	   }
	}
	else {
	   for (i = ileft; i < iright; i+=iskip) {
	    	xb = flic->pic_left + xc + i * sx;
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, yb, xb, yb-6);
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, ye, xb, ye+6);
	   }
	   for (i = ileft; i <= iright; i+=iskip_ch) {
	    	xb = flic->pic_left + xc + i * sx;
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, yb, xb, yb-12);
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, ye, xb, ye+12);
		y = yb +
		   (flic->font_str->ascent + flic->font_str->descent);
		len = sprintf(string, "%d", i);
		ch_wid = XTextWidth(flic->font_str, string, len);
	    	xb = xb - ch_wid/2 ;
		XDrawString(XtDisplay(flic->widget), pixmap,
			gc, xb, y, string, len);
	   }
	}
/* Y or vertical axis */
	xb = flic->pic_left;
	xe = xb + width;
	for(j = jbot; j < jtop ; j+=jskip ) {
	    	yb = flic->height - (flic->pic_bottom + yc + j * sy);
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, yb, xb+6, yb);
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xe, yb, xe-6, yb);
	}
	
	x = xb - del_str;
	ch_wid = flic->font_str->ascent / 2; /* actaully height here...*/
	for(j = jbot; j <= jtop; j+=jskip_ch ) {
	    	yb = flic->height - (flic->pic_bottom + yc + j * sy);
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xb, yb, xb+12, yb);
		XDrawLine(XtDisplay(flic->widget), pixmap, gc,
			xe, yb, xe-12, yb);
	    	yb = yb + ch_wid;
	     	len = sprintf(string, "%d\n", j);
		XDrawString(XtDisplay(flic->widget), pixmap,
			gc, x, yb, string, len);
	}

}

/* Stuff for fonts, cribbed from O'Reilly vol 1, Chap 3, example 3-18,
 * but modified for a font i want. need to learn how to do from resources...
 should change to pass fontname as argument, so main can load from resources */
 
int load_font(dpy, font_info)
Display *dpy;
XFontStruct **font_info;
{
	char *fontname = "timi24";
	
	if( (*font_info = XLoadQueryFont(dpy, fontname))== NULL)
	{
		printf(" cannot open %s font \n", fontname);
		return(-1);
	}
	return(0);
}

/* ARGSUSED */	
void shift_center(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;
{
/* mod from example 15-10 in O'reilly vol 6, page 502 text_box.c 
 * and example 7-2, help_text.c
 * note that shift_label is a Global widget..so callback can change its
 * label string. There must be a better way to do this.
 */
	struct chitra *flic = client_data;
 	char *labels[ ] = {"X shift (km):", "Y shift (km):"}; 
	Widget pb, text_w, form, rowcol, dyalog, pane;
	int i;
	char buf[16];	
	void sc_switch(), get_rmin(), set_result();
	
	dyalog = XtVaCreatePopupShell("Shift",
		xmDialogShellWidgetClass, XtParent(w),
		XmNdeleteResponse, XmDESTROY,
		NULL);
	/* and a place to put... */
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		dyalog, NULL);
		
	rowcol = XtVaCreateWidget("rowcol", xmRowColumnWidgetClass, 
		pane, NULL);
		
/* on-off switch for center shifting */	
	if(flic->shift_center) 
		sprintf(buf,"Center Shifted\n");
	else 
		sprintf(buf,"Not Shifted\n");
		
	shift_label = XtVaCreateManagedWidget("Center", 
		xmPushButtonWidgetClass, rowcol, NULL);
	XtAddCallback(shift_label, XmNactivateCallback, sc_switch, flic);
	XtVaSetValues( shift_label, XtVaTypedArg, XmNlabelString, 
		XmRString, buf, strlen(buf), NULL);

/*		get_rmin(flic->head->idim, flic->head->jdim, 
		  -flic->head->xz + flic->cen_shift[0], 
		  -flic->head->yz + flic->cen_shift[1], 
		  flic->head->sxt, flic->head->syt, &flic->rad_min, &flic->rad_max,
		  &flic->az_min, &flic->az_max); */
	for (i = 0; i < XtNumber(labels); i++) {
		form = XtVaCreateWidget("form", xmFormWidgetClass,
			rowcol, XmNfractionBase, 10, NULL);
		XtVaCreateManagedWidget(labels[i],
			xmLabelGadgetClass, form,
			XmNtopAttachment, XmATTACH_FORM,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNleftAttachment, XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_POSITION,
			XmNrightPosition, 3,
			XmNalignment, XmALIGNMENT_END, NULL);
			
		text_w = XtVaCreateManagedWidget("text_w",
			xmTextFieldWidgetClass, form,
			XmNtraversalOn, True,
			XmNleftAttachment, XmATTACH_POSITION,
			XmNrightAttachment, XmATTACH_FORM,
			XmNleftPosition, 4,
			NULL);
		/* load current value in text widget */
		sprintf(buf,"%.1f",flic->cen_shift[i]);
		XmTextSetString(text_w, buf);
		/* when user hits return, set value in chitra */
		XtAddCallback(text_w, XmNactivateCallback, set_result,
			&(flic->cen_shift[i]) );
		/* and advance focus to next text widget */
		XtAddCallback(text_w, XmNactivateCallback,
			(XtCallbackProc) XmProcessTraversal, 
			(XtPointer) XmTRAVERSE_NEXT_TAB_GROUP);
			
		XtManageChild(form);
	}
	XtManageChild(rowcol);
	XtManageChild(pane);
	XtPopup( dyalog, XtGrabNone);
}

void set_result(text_w, changed_val)
Widget text_w;
float *changed_val;
{
	char *value = XmTextFieldGetString(text_w);
/*	printf(" Value was  %f \n", *changed_val); 
 */
	sscanf(value,"%f", changed_val);
/*	printf(" Value changed to  %f \n", *changed_val);
 */
	XtFree(value);
}


void set_int_result(text_w, changed_val)
Widget text_w;
int *changed_val;
{
	char *value = XmTextFieldGetString(text_w);
/*	printf(" Value was  %d \n", *changed_val); 
 */
	sscanf(value,"%d", changed_val);
/*	printf(" Value changed to  %d \n", *changed_val);
 */
	XtFree(value);
}

/* ARGSUSED */	
void sc_switch(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;
{
	struct chitra *flic = client_data;
	void get_rmin();
	char buf[16];	
        float xz_list;
	if(flic->shift_center) { /* then switch to not shift center */
		flic->shift_center = 0;
		sprintf(buf,"Not Shifted\n");
		get_rmin(flic->head->idim, flic->head->jdim, 
		  -flic->head->xz, 
		  -flic->head->yz, 
		  flic->head->sxt, flic->head->syt, &flic->rad_min, &flic->rad_max,
		  &flic->az_min, &flic->az_max); 
	}
	else {
		flic->shift_center = 1;
		sprintf(buf,"Center Shifted\n");
		get_rmin(flic->head->idim, flic->head->jdim, 
		  -flic->head->xz + flic->cen_shift[0], 
		  -flic->head->yz + flic->cen_shift[1], 
		  flic->head->sxt, flic->head->syt, &flic->rad_min, &flic->rad_max,
		  &flic->az_min, &flic->az_max); 
	}
	XtVaSetValues( shift_label, XtVaTypedArg, XmNlabelString, 
		XmRString, buf, strlen(buf), NULL);
	return;
}

/* ARGSUSED */	
void mr_switch(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;
{
	struct chitra *flic = client_data;
	char buf[16];	
	if(flic->remove_motion) {
		flic->remove_motion = 0;
		sprintf(buf,"Not Removed\n");
	}
	else {
		flic->remove_motion = 1;
		sprintf(buf,"Motion Removed\n");
	}
	XtVaSetValues( motion_label, XtVaTypedArg, XmNlabelString, 
		XmRString, buf, strlen(buf), NULL);
	return;
}

/* ARGSUSED */	
void rem_motion(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;
{
	struct chitra *flic = client_data;

	char *labels[ ] = {"Storm u (m/s):", "Storm v (m/s):"};
	Widget text_w, form, rowcol, dyalog, pane;
	int i;
	char buf[16];	
	void mr_switch(), set_result();
	
	dyalog = XtVaCreatePopupShell("Motion",
		xmDialogShellWidgetClass, XtParent(w),
		XmNdeleteResponse, XmDESTROY,
		NULL);
	/* and a place to put... */
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		dyalog, NULL);
		
	rowcol = XtVaCreateWidget("rowcol", xmRowColumnWidgetClass, 
		pane, NULL);

/* on-off switch for motion removal */	
	if(flic->remove_motion) 
		sprintf(buf,"Motion Removed\n");
	else 
		sprintf(buf,"Not Removed\n");
		
	motion_label = XtVaCreateManagedWidget("MotionLabel", 
		xmPushButtonWidgetClass, rowcol, NULL);
	XtAddCallback(motion_label, XmNactivateCallback, mr_switch, flic);
	XtVaSetValues( motion_label, XtVaTypedArg, XmNlabelString, 
		XmRString, buf, strlen(buf), NULL);

	for (i = 0; i < XtNumber(labels); i++) {
		form = XtVaCreateWidget("form", xmFormWidgetClass,
			rowcol, XmNfractionBase, 10, NULL);
		XtVaCreateManagedWidget(labels[i],
			xmLabelGadgetClass, form,
			XmNtopAttachment, XmATTACH_FORM,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNleftAttachment, XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_POSITION,
			XmNrightPosition, 3,
			XmNalignment, XmALIGNMENT_END, NULL);
			
		text_w = XtVaCreateManagedWidget("text_w",
			xmTextFieldWidgetClass, form,
			XmNtraversalOn, True,
			XmNleftAttachment, XmATTACH_POSITION,
			XmNrightAttachment, XmATTACH_FORM,
			XmNleftPosition, 4,
			NULL);
		/* load current value in text widget */
		sprintf(buf,"%.1f",flic->storm_uv[i]);
		XmTextSetString(text_w, buf);
		/* when user hits return, set value in chitra */
		XtAddCallback(text_w, XmNactivateCallback, set_result,
			&(flic->storm_uv[i]) );
		/* and advance focus to next text widget */
		XtAddCallback(text_w, XmNactivateCallback,
			(XtCallbackProc) XmProcessTraversal, 
			(XtPointer) XmTRAVERSE_NEXT_TAB_GROUP);
			
		XtManageChild(form);
	}
	XtManageChild(rowcol);
	XtManageChild(pane);
	XtPopup( dyalog, XtGrabNone);
}

/* ARGSUSED */	
void value_scaling(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;
{
	struct chitra *flic = client_data;

	char *labels[ ] = {"Min:", "Max:"};
	Widget text_w, form, rowcol, dyalog, pane, var_label;
	int i;
	char buf[20];
	void set_result();
	
	if( flic->var_type == REFLECTIVITY)
		sprintf(buf,"Reflectivity\n");
	else if( flic->var_type == WINDSPEED)
		sprintf(buf," Wind speed\n");
	else if( flic->var_type == VERTWINDSPEED)
		sprintf(buf, "Vertical Velocity\n");
	else if( flic->var_type == RADWIND) 
		sprintf(buf,"Radial Velocity\n");
	else if( flic->var_type == TANWIND) 
		sprintf(buf, "Tangential Velocity\n");
	else if( flic->var_type == U_COMPONENT) 
		sprintf(buf, "U component \n");
	else if( flic->var_type == V_COMPONENT) 
		sprintf(buf, "V component \n");

	dyalog = XtVaCreatePopupShell("Scaling",
		xmDialogShellWidgetClass, XtParent(w),
		XmNdeleteResponse, XmDESTROY,
		NULL);
	/* and a place to put... */
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		dyalog, NULL);
		
	rowcol = XtVaCreateWidget("rowcol", xmRowColumnWidgetClass, 
		pane, NULL);

	var_label = XtVaCreateManagedWidget("var_label",
		xmLabelGadgetClass, rowcol, 
		XtVaTypedArg, XmNlabelString, XmRString,buf, strlen(buf),
		NULL);

	for (i = 0; i < XtNumber(labels); i++) {
		form = XtVaCreateWidget("form", xmFormWidgetClass,
			rowcol, XmNfractionBase, 10, NULL);
		XtVaCreateManagedWidget(labels[i],
			xmLabelGadgetClass, form,
			XmNtopAttachment, XmATTACH_FORM,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNleftAttachment, XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_POSITION,
			XmNrightPosition, 3,
			XmNalignment, XmALIGNMENT_END, NULL);
			
		text_w = XtVaCreateManagedWidget("text_w",
			xmTextFieldWidgetClass, form,
			XmNtraversalOn, True,
			XmNleftAttachment, XmATTACH_POSITION,
			XmNrightAttachment, XmATTACH_FORM,
			XmNleftPosition, 4,
			NULL);
		/* load current value into window */
		sprintf(buf,"%.1f",
			flic->var_scale[flic->var_type][i]);
                XmTextSetString(text_w, buf);
		/* when user hits return, set value in chitra */
		XtAddCallback(text_w, XmNactivateCallback, set_result,
			&(flic->var_scale[flic->var_type][i]) );
		/* and advance focus to next text widget */
		XtAddCallback(text_w, XmNactivateCallback,
			(XtCallbackProc) XmProcessTraversal, 
			(XtPointer) XmTRAVERSE_NEXT_TAB_GROUP);
			
		XtManageChild(form);
	}
	XtManageChild(rowcol);
	XtManageChild(pane);
	XtPopup( dyalog, XtGrabNone);
}
/* ARGSUSED */	
void geometry_scaling(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;
{
	struct chitra *flic = client_data;

	char *labels[ ] = {"I min","I max", "J min","J max", "K min","K max",
		"R min (km)", "R max (km)"};
	Widget text_w, form, rowcol, dyalog, pane, var_label;
	int i;
	char buf[20];
	void set_int_result(), set_result();
	
	sprintf(buf,"Array Window\n");
	
	dyalog = XtVaCreatePopupShell("Scaling",
		xmDialogShellWidgetClass, XtParent(w),
		XmNdeleteResponse, XmDESTROY,
		NULL);
	/* and a place to put... */
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		dyalog, NULL);
		
	rowcol = XtVaCreateWidget("rowcol", xmRowColumnWidgetClass, 
		pane, NULL);

	var_label = XtVaCreateManagedWidget("var_label",
		xmLabelGadgetClass, rowcol, 
		XtVaTypedArg, XmNlabelString, XmRString,buf, strlen(buf),
		NULL);

	for (i = 0; i < XtNumber(labels); i++) {
		form = XtVaCreateWidget("form", xmFormWidgetClass,
			rowcol, XmNfractionBase, 10, NULL);
		XtVaCreateManagedWidget(labels[i],
			xmLabelGadgetClass, form,
			XmNtopAttachment, XmATTACH_FORM,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNleftAttachment, XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_POSITION,
			XmNrightPosition, 3,
			XmNalignment, XmALIGNMENT_END, NULL);
			
		text_w = XtVaCreateManagedWidget("text_w",
			xmTextFieldWidgetClass, form,
			XmNtraversalOn, True,
			XmNleftAttachment, XmATTACH_POSITION,
			XmNrightAttachment, XmATTACH_FORM,
			XmNleftPosition, 4,
			NULL);
		/* when user hits return, set value in chitra */
		if( i == 0 )
		  XtAddCallback(text_w, XmNactivateCallback, 
			(XtCallbackProc) set_int_result,
			&(flic->iw[0]) );
		else if( i == 1 )
		  XtAddCallback(text_w, XmNactivateCallback, 
			(XtCallbackProc) set_int_result,
			&(flic->iw[1]) );
		else if( i == 2 )
		  XtAddCallback(text_w, XmNactivateCallback, 
			(XtCallbackProc) set_int_result,
			&(flic->jw[0]) );
		else if( i == 3 )
		  XtAddCallback(text_w, XmNactivateCallback, 
			(XtCallbackProc) set_int_result,
			&(flic->jw[1]) );
		else if( i == 4 )
		  XtAddCallback(text_w, XmNactivateCallback, 
			(XtCallbackProc) set_int_result,
			&(flic->kw[0]) );
		else if( i == 5 )
		  XtAddCallback(text_w, XmNactivateCallback, 
			(XtCallbackProc) set_int_result,
			&(flic->kw[1]) );
		else if( i == 6 )
		  XtAddCallback(text_w, XmNactivateCallback, 
			(XtCallbackProc) set_result,
			&(flic->rad_min) );
		else if( i == 7 )
		  XtAddCallback(text_w, XmNactivateCallback, 
			(XtCallbackProc) set_result,
			&(flic->rad_max) );
		/* and advance focus to next text widget */
		XtAddCallback(text_w, XmNactivateCallback,
			(XtCallbackProc) XmProcessTraversal, 
			(XtPointer) XmTRAVERSE_NEXT_TAB_GROUP);
			
		XtManageChild(form);
	}
	XtManageChild(rowcol);
	XtManageChild(pane);
	XtPopup( dyalog, XtGrabNone);
}

/* ARGSUSED */	
void arrow_scaling(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;
{
	struct chitra *flic = client_data;

	char *labels[ ] = {"Width:", "Length:"};
	Widget text_w, form, rowcol, dyalog, pane, var_label;
	int i;
	char buf[20];
	void set_int_result();
	
	sprintf(buf, "Arrow Scaling \n");

	dyalog = XtVaCreatePopupShell("Arrows",
		xmDialogShellWidgetClass, XtParent(w),
		XmNdeleteResponse, XmDESTROY,
		NULL);
	/* and a place to put... */
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		dyalog, NULL);
		
	rowcol = XtVaCreateWidget("rowcol", xmRowColumnWidgetClass, 
		pane, NULL);

	var_label = XtVaCreateManagedWidget("var_label",
		xmLabelGadgetClass, rowcol, 
		XtVaTypedArg, XmNlabelString, XmRString,buf, strlen(buf),
		NULL);

		form = XtVaCreateWidget("form", xmFormWidgetClass,
			rowcol, XmNfractionBase, 10, NULL);
		XtVaCreateManagedWidget("Shaft width",
			xmLabelGadgetClass, form,
			XmNtopAttachment, XmATTACH_FORM,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNleftAttachment, XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_POSITION,
			XmNrightPosition, 3,
			XmNalignment, XmALIGNMENT_END, NULL);
			
		text_w = XtVaCreateManagedWidget("text_w",
			xmTextFieldWidgetClass, form,
			XmNtraversalOn, True,
			XmNleftAttachment, XmATTACH_POSITION,
			XmNrightAttachment, XmATTACH_FORM,
			XmNleftPosition, 4,
			NULL);
		/* load current value into window */
		sprintf(buf,"%d",flic->arrow_shaft_width);
                XmTextSetString(text_w, buf);
		/* when user hits return, set value in chitra */
		XtAddCallback(text_w, XmNactivateCallback, 
			(XtCallbackProc) set_int_result,
			&(flic->arrow_shaft_width) );
		/* and advance focus to next text widget */
		XtAddCallback(text_w, XmNactivateCallback,
			(XtCallbackProc) XmProcessTraversal, 
			(XtPointer) XmTRAVERSE_NEXT_TAB_GROUP);
			
		XtManageChild(form);

		form = XtVaCreateWidget("form", xmFormWidgetClass,
			rowcol, XmNfractionBase, 10, NULL);
		XtVaCreateManagedWidget("Shaft Length",
			xmLabelGadgetClass, form,
			XmNtopAttachment, XmATTACH_FORM,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNleftAttachment, XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_POSITION,
			XmNrightPosition, 3,
			XmNalignment, XmALIGNMENT_END, NULL);
			
		text_w = XtVaCreateManagedWidget("text_w",
			xmTextFieldWidgetClass, form,
			XmNtraversalOn, True,
			XmNleftAttachment, XmATTACH_POSITION,
			XmNrightAttachment, XmATTACH_FORM,
			XmNleftPosition, 4,
			NULL);
		/* load current value into window */
		sprintf(buf,"%d",flic->arrow_shaft_length);
                XmTextSetString(text_w, buf);
		/* when user hits return, set value in chitra */
		XtAddCallback(text_w, XmNactivateCallback, 
			(XtCallbackProc) set_int_result,
			&(flic->arrow_shaft_length) );
		/* and advance focus to next text widget */
		XtAddCallback(text_w, XmNactivateCallback,
			(XtCallbackProc) XmProcessTraversal, 
			(XtPointer) XmTRAVERSE_NEXT_TAB_GROUP);
			
		XtManageChild(form);


		form = XtVaCreateWidget("form", xmFormWidgetClass,
			rowcol, XmNfractionBase, 10, NULL);
		XtVaCreateManagedWidget("Plot Interval",
			xmLabelGadgetClass, form,
			XmNtopAttachment, XmATTACH_FORM,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNleftAttachment, XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_POSITION,
			XmNrightPosition, 3,
			XmNalignment, XmALIGNMENT_END, NULL);
			
		text_w = XtVaCreateManagedWidget("text_w",
			xmTextFieldWidgetClass, form,
			XmNtraversalOn, True,
			XmNleftAttachment, XmATTACH_POSITION,
			XmNrightAttachment, XmATTACH_FORM,
			XmNleftPosition, 4,
			NULL);
		/* load current value into window */
		sprintf(buf,"%d",flic->arrow_interval);
                XmTextSetString(text_w, buf);
		/* when user hits return, set value in chitra */
		XtAddCallback(text_w, XmNactivateCallback, 
			(XtCallbackProc) set_int_result,
			&(flic->arrow_interval) );
		/* and advance focus to next text widget */
		XtAddCallback(text_w, XmNactivateCallback,
			(XtCallbackProc) XmProcessTraversal, 
			(XtPointer) XmTRAVERSE_NEXT_TAB_GROUP);
			
		XtManageChild(form);

	XtManageChild(rowcol);
	XtManageChild(pane);
	XtPopup( dyalog, XtGrabNone);
}
/* ARGSUSED */	
void get_rmin(idim, jdim, xs,ys, sx, sy, rmin, rmax, az_min, az_max)
int idim, jdim;
float xs, ys, sx, sy;
float *rmax, *rmin, *az_min, *az_max;
/*
 * 	Given box idim*sx by jdim*sy, determine min and max distance from 
 * 0,0 where lower left corner of box is located at xs,ys.
 * also determine bracketing azimuths, in math angles, return in radians.
 */
{
 int i, inside;
 float xr, yt, r, swap;
 
 printf("get_min entered \n");
 xr = xs + idim * sx;
 yt = ys + jdim * sy;
/* Find where 0,0 lies in relation to box, using same logic as in 
 * clipping a line to a window. 
 *     1001/9  1000/8  1010/10    above the window 
 *     0001/1  0000/0  0010/2     left,window,right
 *     0101/5  0100/4  0110/6     below the window 
 */
	inside = 0;
	if( 0.0 < xs)
              inside = inside + 1;
	else if( 0.0 > xr) 
              inside = inside + 2; 
	if(0.0 < ys)
              inside = inside + 4;
	else if(0.0 > yt) 
              inside = inside + 8;
               
 	if( inside == 0 ) {
		*rmin = 0.0;
		*rmax = xs*xs + ys*ys;
		r = xr*xr + ys*ys;
		if(r > *rmax ) *rmax = r;
		r = xr*xr + yt*yt;
		if(r > *rmax ) *rmax = r;
		r = xs*xs + yt*yt;
		if(r > *rmax ) *rmax = r;
		*rmax = sqrt( (double) *rmax);
/* Buncha code here because I want to take care of cases where (0,0)
 * lies on edge of box, so I do not return 0 to 2 pi for those */
 		if( xs == 0.0 ) {
 			if( ys == 0.0 ) {
 				*az_min = 0.0;
 				*az_max = 0.5 * acos(-1.0);
 			}
 			else if( yt == 0.0 ) {
 				*az_min = 1.5 * acos(-1.0);
				*az_max = 2.0 * acos(-1.0);
			}
			else {
 				*az_min = 1.5 * acos(-1.0);
				*az_max = 2.5 * acos(-1.0);
			}
		}
 		else if( xr == 0.0 ) {
 			if( ys == 0.0 ) {
 				*az_min = 0.5 * acos(-1.0);
 				*az_max = acos(-1.0);
 			}
 			else if( yt == 0.0 ) {
 				*az_min = acos(-1.0);
				*az_max = 1.5 * acos(-1.0);
			}
			else {
 				*az_min = 0.5 * acos(-1.0);
				*az_max = 1.5 * acos(-1.0);
			}
		}
		else if ( ys == 0.0 ) {
			*az_min = 0.0;
			*az_max = acos(-1.0);
		}
		else if ( yt == 0.0 ) {
			*az_min = acos(-1.0);
			*az_max = 2.0 * acos(-1.0);
		}
		else { /* the most usual case! */
			*az_min = 0.0;
			*az_max = 2.0 * acos(-1.0);
		}
	}
	else if( inside == 1 ) {
		*rmin = xs;
		*rmax = xr;
		*az_min = atan2(ys, xs);
		*az_max = atan2(yt, xs);
		if( *az_max < *az_min ) {
			swap = *az_max;
			*az_max = *az_min;
			*az_min = swap;
		}
	}
	else if( inside == 2 ) {
		*rmin = -xr;
		*rmax = -xs;
		*az_min = atan2(ys, xr);
		*az_max = atan2(yt, xr);
		if( *az_max < *az_min ) {
			swap = *az_max;
			*az_max = *az_min;
			*az_min = swap;
		}
	}
	else if( inside == 4 ) {
		*rmin = ys;
		*rmax = yt;
		*az_min = atan2(ys, xs);
		*az_max = atan2(ys, xr);
		if( *az_max < *az_min ) {
			swap = *az_max;
			*az_max = *az_min;
			*az_min = swap;
		}
	}
	else if( inside == 8 ) {
		*az_min = atan2(yt, xr);
		*az_max = atan2(yt, xs);
		if( *az_max < *az_min ) {
			swap = *az_max;
			*az_max = *az_min;
			*az_min = swap;
		}
		*rmin = -yt;
		*rmax = -ys;
	}
	else if( ( inside == 5 ) || ( inside == 10 ) ){
		*rmin = sqrt(xs*xs + yt*yt);
		*rmax = sqrt(xr*xr + ys*ys);
		if( *rmin > *rmax ) {
			swap = *rmax;
			*rmax = *rmin;
			*rmin = swap;
		}
		*az_min = atan2(yt, xs);
		*az_max = atan2(ys, xr);
		if( *az_max < *az_min ) {
			swap = *az_max;
			*az_max = *az_min;
			*az_min = swap;
		}
	}
	else if( ( inside == 6 ) || ( inside == 9 ) ) {
		*rmin = sqrt(xs*xs + ys*ys);
		*rmax = sqrt(xr*xr + yt*yt);
		if( *rmin > *rmax ) {
			swap = *rmax;
			*rmax = *rmin;
			*rmin = swap;
		}
		*az_min = atan2(ys, xs);
		*az_max = atan2(yt, xr);
		if( *az_max < *az_min ) {
			swap = *az_max;
			*az_max = *az_min;
			*az_min = swap;
		}
	}
/* then code to check for az diff < 90... */
/* Debug:
 */
	printf(" xs, ys, xr, yt: %f %f %f %f \n",xs, ys, xr, yt);
	printf("Get_rmin finds rmin %f and rmax %f \n", *rmin, *rmax);
	printf("Get_rmin finds az min %f and max %f \n", *az_min, *az_max);
 return;
}
/* ARGSUSED */	
void set_ring_frac(w, client_data, call_data)
Widget w;
XtPointer client_data /* to be cast to structure */;
XtPointer call_data;
{
	struct chitra *flic = client_data;
	Widget text_w, form, rowcol, dyalog, pane, var_label;
	char buf[25];
	void set_float_result();
	dyalog = XtVaCreatePopupShell("Ring Fraction",
		xmDialogShellWidgetClass, XtParent(w),
		XmNdeleteResponse, XmDESTROY,
		NULL);
	/* and a place to put... */
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		dyalog, NULL);
		
	rowcol = XtVaCreateWidget("rowcol", xmRowColumnWidgetClass, 
		pane, NULL);

	sprintf(buf, "Set Ring fraction (0 - 1)\n");
	var_label = XtVaCreateManagedWidget("var_label",
		xmLabelGadgetClass, rowcol, 
		XtVaTypedArg, XmNlabelString, XmRString,buf, strlen(buf),
		NULL);

	form = XtVaCreateWidget("form", xmFormWidgetClass,
			rowcol, XmNfractionBase, 10, NULL);
	sprintf(buf, "Fraction: \n");
	XtVaCreateManagedWidget("Fraction",
		xmLabelGadgetClass, form,
		XtVaTypedArg, XmNlabelString, XmRString,buf, strlen(buf),
		XmNtopAttachment, XmATTACH_FORM,
		XmNbottomAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 3,
		XmNalignment, XmALIGNMENT_END, NULL);
	text_w = XtVaCreateManagedWidget("text_w",
		xmTextFieldWidgetClass, form,
		XmNtraversalOn, True,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNrightAttachment, XmATTACH_FORM,
		XmNleftPosition, 4,
		NULL);
	/* load current value in text widget */
	sprintf(buf,"%.2f",flic->arc_min_frac);
	XmTextSetString(text_w, buf);
	/* when user hits return, set value in chitra */
	XtAddCallback(text_w, XmNactivateCallback, 
	(XtCallbackProc) set_result, &(flic->arc_min_frac) );
	XtManageChild(form);
	XtManageChild(rowcol);
	XtManageChild(pane);
	XtPopup( dyalog, XtGrabNone);
}



/* Event handler for button press or release in widget */
void mouser(w, client_data, event)
Widget w;
XtPointer client_data /* to be cast to structure */;
XButtonEvent *event; /* I cast specifically, because mask only selects these..*/
{
	/* cast required in ANSI C , but I could not get to work if
	 * cast the way they did in O'Reilly....*/
	struct chitra *flic = client_data;
	static Position x1, y1, x2, y2 ; /* points on screen */
	static int i1, i2, j1, j2; /* points in array */
	static int first_point = 1;
	int window_to_data();
	float x, y;
	double shaft;
	int i,j,k, sign;
	int wpix, hpix;
 if( flic->orientation == ARB_SLICE ) { /* use mouse to select slice*/
	if(event->type == ButtonPress) {
	     if( first_point ) {  
		XDrawLine(event->display, event->window, gc, 
		 event->x+5, event->y, event->x-5, event->y);
		XDrawLine(event->display, event->window, gc, 
		 event->x, event->y+5, event->x, event->y-5);
		x1 = event->x;
		y1 = event->y;
		x2 = x1;
		y2 = y1;
		first_point = 0;
/* convert to location in array */
	   	if( window_to_data(event->x, event->y, &i1, &j1, &k, &x, &y,
	   		 flic) == 0) {
	   	  printf(" Beginning point i %d, j %d \n",i1,j1);
	   	  flic->x_beg = x;
	   	  flic->y_beg = y;
		}
/* Set to XOR drawing for rubber banding of line:
 */
	XSetFunction(event->display, gc, GXxor);
/* Have to remember to turn this OFF after, or vector graphics may get 
 * screwed up !
 */
	     }
	}
	else if( event->type == ButtonRelease ) { /* other corner */
	   	if( window_to_data(event->x, event->y, &i2, &j2, &k, &x, &y,
	   		 flic) == 0) {
	   	  printf(" end point: i %d, j %d \n",i2,j2);
	   	  flic->x_end = x;
	   	  flic->y_end = y;
		}
		x2 = event->x;
		y2 = event->y;
		XDrawLine(event->display, event->window, gc, 
		 event->x+5, event->y, event->x-5, event->y);
		XDrawLine(event->display, event->window, gc, 
		 event->x, event->y+5, event->x, event->y-5);
/* turn off XOR drawing 
 */

		XSetFunction(event->display, gc, GXcopy);
	    /* draw line */
		XDrawLine(event->display, event->window, gc, x1,y1,
		 x2, y2);
		first_point = 1; /* reset for next slice */
	}
	else if( event->type == MotionNotify ) { /* draw rubber line */
	/* only draw rubber line if we are doing selection...*/
	   if( first_point == 0 ) 
	   {
		XDrawLine(event->display, event->window, gc, x1,y1,
		 x2, y2);
		x2 = event->x;
		y2 = event->y;
/*
		printf(" MotionNotify x %d y %d \n",x2, y2);
 */
		XDrawLine(event->display, event->window, gc, x1,y1,
		 x2, y2);
	    }
	}
  }
}

int window_to_data(x, y, i, j, k, xt, yt, flic)
/* Convert screen coordinates to nearest location in data array.
 * Also return float coordinates of the array point.
 * math is sort of messy, because no nint function, plus have to careful
 * about going between int and float, hence the intermediate variables
 * dx, dy, xt, and yt.
 */
int x, y, *i, *j, *k;
float *xt, *yt; /* float grid coords, so can find nearest array index */
struct chitra *flic;
{
	double dx, dy; /* float offsets so divide by nzx etc. is cool */ 
	Position xp, yp;
			
	if( ( flic->orientation == XY_SLICE) || 
		( flic->orientation == ARB_SLICE) )
	{ /* XY slice or need pos in xy slice for arbitrary slice */
	  *k = flic->level;
 	  dx = (x - flic->pic_left );
	  *xt = flic->iw[0] + dx / flic->nzx;
	  dy = (flic->height - y - flic->pic_bottom);
	  *yt = flic->jw[0] + dy / flic->nzy;
	  *i = floor(*xt);
	  if( fabs( *i - *xt) > 0.5 )
	  	*i = ceil(*xt);
	  *j = floor(*yt);
	  if( fabs( *j - *yt) > 0.5 )
	  	*j = ceil(*yt);
	  	
	  xp = flic->pic_left + (*i - flic->iw[0]) * flic->nzx;
	  yp = flic->height - (  flic->pic_bottom + (*j - flic->jw[0]) 
	  	* flic->nzy);

	  if( (*i < flic->idim) && (*i > -1) && ( *j < flic->jdim) &&
	  	(*j > -1) ) 
	  	return 0;
	  else
	  	return 1; /* Out of bounds */
	}
	else if ( flic->orientation == XZ_SLICE) 
	{ 
	  *j = flic->level;
 	  dx = (x - flic->pic_left );
	  *xt = flic->iw[0] + dx / flic->nzx;
	  dy = (flic->height - y - flic->pic_bottom);
	  *yt = flic->kw[0] + dy / flic->nzy;
	  *i = floor(*xt);
	  if( fabs( *i - *xt) > 0.5 )
	  	*i = ceil(*xt);
	  *k = floor(*yt);
	  if( fabs( *k - *yt) > 0.5 )
	  	*k = ceil(*yt);
	  xp = flic->pic_left + (*i - flic->iw[0]) * flic->nzx;
	  yp = flic->height - (  flic->pic_bottom + (*k - flic->kw[0]) 
	  	* flic->nzy);
	  if( (*i < flic->idim) && (*i > -1) && ( *k < flic->kdim) &&
	  	(*k > -1) ) 
	  	return 0;
	  else
	  	return 1; /* Out of bounds */
	}
	else if ( flic->orientation == YZ_SLICE) 
	{ 
	  *i = flic->level;
 	  dx = (x - flic->pic_left );
	  *xt = flic->jw[0] + dx / flic->nzx;
	  dy = (flic->height - y - flic->pic_bottom);
	  *yt = flic->kw[0] + dy / flic->nzy;
	  *j = floor(*xt);
	  if( fabs( *j - *xt) > 0.5 )
	  	*j = ceil(*xt);
	  *k = floor(*yt);
	  if( fabs( *k - *yt) > 0.5 )
	  	*k = ceil(*yt);

	  xp = flic->pic_left + (*j - flic->jw[0]) * flic->nzx;
	  yp = flic->height - (  flic->pic_bottom + (*k - flic->kw[0]) 
	  	* flic->nzy);
	  if( (*k < flic->kdim) && (*k > -1) && ( *j < flic->jdim) &&
	  	(*j > -1) ) 
	  	return 0;
	  else
	  	return 1; /* Out of bounds */
	}

}

